//---------------------------------------------------------------------------
#pragma hdrstop

#include <Math.h>

#include "Curva.h"
#include "Reta.h"
#include "Ponto.h"
#include "Matriz.h"
//---------------------------------------------------------------------------
Curva::Curva(string nome, tipoCurva _tipo) :
  ObjetoGrafico(nome),
  retasDaCurva(NULL)
{
  tipo = _tipo;
  matrizDaCurva = MatrizDaCurva(tipo);
}
//---------------------------------------------------------------------------
Matriz* Curva::MatrizDaCurva(tipoCurva tipoDaCurva)
{
  Matriz* matrizDaCurva = new Matriz(4,4);
  switch(tipoDaCurva) {
  case HERMITE:
    matrizDaCurva->ColocarElemento(0,0,2);
    matrizDaCurva->ColocarElemento(0,1,-2);
    matrizDaCurva->ColocarElemento(0,2,1);
    matrizDaCurva->ColocarElemento(0,3,1);
    matrizDaCurva->ColocarElemento(1,0,-3);
    matrizDaCurva->ColocarElemento(1,1,3);
    matrizDaCurva->ColocarElemento(1,2,-2);
    matrizDaCurva->ColocarElemento(1,3,-1); // No pdf do professor est 1
    matrizDaCurva->ColocarElemento(2,0,0);
    matrizDaCurva->ColocarElemento(2,1,0);
    matrizDaCurva->ColocarElemento(2,2,1);
    matrizDaCurva->ColocarElemento(2,3,0);
    matrizDaCurva->ColocarElemento(3,0,1);
    matrizDaCurva->ColocarElemento(3,1,0);
    matrizDaCurva->ColocarElemento(3,2,0);
    matrizDaCurva->ColocarElemento(3,3,0);
    break;
  case BEZIER:
    matrizDaCurva->ColocarElemento(0,0,-1);
    matrizDaCurva->ColocarElemento(0,1,3);
    matrizDaCurva->ColocarElemento(0,2,-3);
    matrizDaCurva->ColocarElemento(0,3,1);
    matrizDaCurva->ColocarElemento(1,0,3);
    matrizDaCurva->ColocarElemento(1,1,-6);
    matrizDaCurva->ColocarElemento(1,2,3);
    matrizDaCurva->ColocarElemento(1,3,0);
    matrizDaCurva->ColocarElemento(2,0,-3);
    matrizDaCurva->ColocarElemento(2,1,3);
    matrizDaCurva->ColocarElemento(2,2,0);
    matrizDaCurva->ColocarElemento(2,3,0);
    matrizDaCurva->ColocarElemento(3,0,1);
    matrizDaCurva->ColocarElemento(3,1,0);
    matrizDaCurva->ColocarElemento(3,2,0);
    matrizDaCurva->ColocarElemento(3,3,0);
    break;
  case SPLINE:
    matrizDaCurva->ColocarElemento(0,0,-(1.0/6.0));
    matrizDaCurva->ColocarElemento(0,1,1.0/2.0);
    matrizDaCurva->ColocarElemento(0,2,-(1.0/2.0));
    matrizDaCurva->ColocarElemento(0,3,1.0/6.0);
    matrizDaCurva->ColocarElemento(1,0,1.0/2.0);
    matrizDaCurva->ColocarElemento(1,1,-1.0);
    matrizDaCurva->ColocarElemento(1,2,1.0/2.0);
    matrizDaCurva->ColocarElemento(1,3,0.0);
    matrizDaCurva->ColocarElemento(2,0,-(1.0/2.0));
    matrizDaCurva->ColocarElemento(2,1,0.0);
    matrizDaCurva->ColocarElemento(2,2,1.0/2.0);
    matrizDaCurva->ColocarElemento(2,3,0.0);
    matrizDaCurva->ColocarElemento(3,0,1.0/6.0);
    matrizDaCurva->ColocarElemento(3,1,2.0/3.0);
    matrizDaCurva->ColocarElemento(3,2,1.0/6.0);
    matrizDaCurva->ColocarElemento(3,3,0.0);
  }
  return matrizDaCurva;
}
//---------------------------------------------------------------------------
Curva::~Curva()
{
  delete matrizDaCurva;
}
//---------------------------------------------------------------------------
void Curva::AdicionaPonto(Ponto* ponto)
{
  pontos.push_back(ponto);
}
//---------------------------------------------------------------------------
Ponto* Curva::Centro()
{
  float dx = 0.0;
  float dy = 0.0;
  float dz = 0.0;
  for (unsigned int i=0;i<pontos.size();i++) {
    Ponto* p = pontos[i];
    dx += p->GetX();
    dy += p->GetY();
    dz += p->GetZ();
  }

  return new Ponto("Centro",dx/pontos.size(),dy/pontos.size(),dz/pontos.size());
}
//---------------------------------------------------------------------------
void Curva::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao, bool CPP)
{
    Ponto* centro = NULL;
    if (!CPP)
      centro = Centro();
    Matriz* matriz = CriarMatrizTransformacoes(translacao,escalonamento,rotacao,centro);
    for (unsigned int i=0;i<pontos.size();i++) {
      Ponto* p = pontos[i];
      AplicaTransformacaoNoPonto(p, matriz,CPP);
    }
    delete matriz;
    delete centro;
}
//---------------------------------------------------------------------------
void Curva::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  for (unsigned int i=0;i<pontos.size();i++) {
    Ponto* p = pontos[i];
    AplicaTransformacaoNoPonto(p, matrizDeTransformacao,CPP);
  }
}
//---------------------------------------------------------------------------
int Curva::PassoRecomendado(float tamWindow, int tamViewport)
{
  float taxaPixels = tamWindow / tamViewport;
  taxaPixels *= 5;
  float dx = 0.0;
  float dy = 0.0;
  for (unsigned int i=0;i<pontos.size()-1;i++) {
    dx += fabs(pontos[i+1]->GetXWindow() - pontos[i]->GetXWindow());
    dy += fabs(pontos[i+1]->GetYWindow() - pontos[i]->GetYWindow());
  }
  int result = max(dx,dy)/taxaPixels;
  if (result > 100)
    result = 100;
  if (result < 10)
    result = 10;

  return result;
}
//---------------------------------------------------------------------------
void Curva::SetPontos(vector<Ponto*>* _pontos)
{
  pontos = *_pontos;
}
//---------------------------------------------------------------------------
vector<float*>* Curva::CoeficientesDaCurva()
// Retorna um array com os coeficientes da curva
// ax, bx, cx, dx, ay, by, cy, dy, az, bz, cz, dz
{
  vector<float*>* retorno = new vector<float*>;
  int incremento;
  if (tipo == BEZIER)
    incremento = 3;
  else if (tipo == SPLINE)
    incremento = 1;
  else if (tipo == HERMITE)
    incremento = 2;

  for(unsigned int i=0;i<pontos.size()-3;i+=incremento){
    float* coeficientes = new float[12];
    Matriz matrizPontosX(4,1);
    if (tipo == HERMITE) {
      matrizPontosX.ColocarElemento(0,0,pontos[i]->GetXWindow());
      matrizPontosX.ColocarElemento(1,0,pontos[i+2]->GetXWindow());
      matrizPontosX.ColocarElemento(2,0,pontos[i+1]->GetXWindow() - pontos[i]->GetXWindow());
      matrizPontosX.ColocarElemento(3,0,pontos[i+2]->GetXWindow() - pontos[i+3]->GetXWindow());
    } else {
      matrizPontosX.ColocarElemento(0,0,pontos[i]->GetXWindow());
      matrizPontosX.ColocarElemento(1,0,pontos[i+1]->GetXWindow());
      matrizPontosX.ColocarElemento(2,0,pontos[i+2]->GetXWindow());
      matrizPontosX.ColocarElemento(3,0,pontos[i+3]->GetXWindow());
    }

    Matriz* coeficientesX = Matriz::MultiplicaMatriz(matrizDaCurva, &matrizPontosX);
    coeficientes[0] = coeficientesX->GetElemento(0,0);
    coeficientes[1] = coeficientesX->GetElemento(1,0);
    coeficientes[2] = coeficientesX->GetElemento(2,0);
    coeficientes[3] = coeficientesX->GetElemento(3,0);

    Matriz matrizPontosY(4,1);
    if (tipo == HERMITE) {
      matrizPontosY.ColocarElemento(0,0,pontos[i]->GetYWindow());
      matrizPontosY.ColocarElemento(1,0,pontos[i+2]->GetYWindow());
      matrizPontosY.ColocarElemento(2,0,pontos[i+1]->GetYWindow() - pontos[i]->GetYWindow());
      matrizPontosY.ColocarElemento(3,0,pontos[i+2]->GetYWindow() - pontos[i+3]->GetYWindow());
    } else {
      matrizPontosY.ColocarElemento(0,0,pontos[i]->GetYWindow());
      matrizPontosY.ColocarElemento(1,0,pontos[i+1]->GetYWindow());
      matrizPontosY.ColocarElemento(2,0,pontos[i+2]->GetYWindow());
      matrizPontosY.ColocarElemento(3,0,pontos[i+3]->GetYWindow());
    }

    Matriz* coeficientesY = Matriz::MultiplicaMatriz(matrizDaCurva, &matrizPontosY);
    coeficientes[4] = coeficientesY->GetElemento(0,0);
    coeficientes[5] = coeficientesY->GetElemento(1,0);
    coeficientes[6] = coeficientesY->GetElemento(2,0);
    coeficientes[7] = coeficientesY->GetElemento(3,0);

    Matriz matrizPontosZ(4,1);
    if (tipo == HERMITE) {
      matrizPontosZ.ColocarElemento(0,0,pontos[i]->GetZWindow());
      matrizPontosZ.ColocarElemento(1,0,pontos[i+2]->GetZWindow());
      matrizPontosZ.ColocarElemento(2,0,pontos[i+1]->GetZWindow() - pontos[i]->GetZWindow());
      matrizPontosZ.ColocarElemento(3,0,pontos[i+2]->GetZWindow() - pontos[i+3]->GetZWindow());
    } else {
      matrizPontosZ.ColocarElemento(0,0,pontos[i]->GetZWindow());
      matrizPontosZ.ColocarElemento(1,0,pontos[i+1]->GetZWindow());
      matrizPontosZ.ColocarElemento(2,0,pontos[i+2]->GetZWindow());
      matrizPontosZ.ColocarElemento(3,0,pontos[i+3]->GetZWindow());
    }

    Matriz* coeficientesZ = Matriz::MultiplicaMatriz(matrizDaCurva, &matrizPontosZ);
    coeficientes[8] = coeficientesZ->GetElemento(0,0);
    coeficientes[9] = coeficientesZ->GetElemento(1,0);
    coeficientes[10] = coeficientesZ->GetElemento(2,0);
    coeficientes[11] = coeficientesZ->GetElemento(3,0);

    delete coeficientesX;
    delete coeficientesY;
    delete coeficientesZ;
    retorno->push_back(coeficientes);
  }
  return retorno;
}
//---------------------------------------------------------------------------
void Curva::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);
  output << tipo << " ";
  output << pontos.size()<< " ";
  for (unsigned int i=0;i<pontos.size();i++)
    pontos[i]->SalvaNoStream(output);

}
//---------------------------------------------------------------------------
Curva* Curva::LeDoStream(istream &input, int versao)
{
  string nome = ObjetoGrafico::LeNomeDoStream(input);
  int tipo;
  input >> tipo;
  Curva* curva = new Curva(nome,(tipoCurva)tipo);
  int nrPontos;
  input >> nrPontos;
  for(int i=0 ;i<nrPontos;i++)
    curva->AdicionaPonto(Ponto::LeDoStream(input, versao));

  return curva;
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
