//---------------------------------------------------------------------------
#pragma hdrstop

#include <math.h>
#include "Matriz.h"
//---------------------------------------------------------------------------
Matriz::Matriz(int _nrLinhas, int _nrColunas)
{
  nrLinhas = _nrLinhas;
  nrColunas = _nrColunas;
  elementos = new float[nrLinhas*nrColunas];
}
//---------------------------------------------------------------------------
Matriz::~Matriz()
{
  delete[] elementos;
}
//---------------------------------------------------------------------------
void Matriz::ColocarElemento(int linha, int coluna, float numero)
{
    elementos[linha*nrColunas + coluna] = numero;
}
//---------------------------------------------------------------------------
int Matriz::GetNrLinhas()
{                               
  return nrLinhas;
}
//---------------------------------------------------------------------------
int Matriz::GetNrColunas()
{
  return nrColunas;
}
//---------------------------------------------------------------------------
float Matriz::GetElemento(int linha, int coluna) const
{
  return elementos[linha*nrColunas + coluna];
}
//---------------------------------------------------------------------------
Matriz* Matriz::MultiplicaMatriz(Matriz* uma, Matriz* outra)
{
  Matriz* retorno = new Matriz(uma->GetNrLinhas(), outra->GetNrColunas());
  if (uma->GetNrColunas() == outra->GetNrLinhas()){
    for(int i=0;i<uma->GetNrLinhas();i++) {
      for(int j=0;j<outra->GetNrColunas();j++){
        double soma =0;
        for(int k=0;k<uma->GetNrColunas();k++){
          soma += uma->GetElemento(i,k) * outra->GetElemento(k,j);
        }
        retorno->ColocarElemento(i,j,soma);
      }
    }
  }
  return retorno;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizTransformacaoTranslacao(float* translacao)
{
  Matriz* temp = new Matriz(4,4);
  temp->ColocarElemento(0,0,1.0);
  temp->ColocarElemento(0,1,0);
  temp->ColocarElemento(0,2,0);
  temp->ColocarElemento(0,3,0);
  temp->ColocarElemento(1,0,0);
  temp->ColocarElemento(1,1,1.0);
  temp->ColocarElemento(1,2,0);
  temp->ColocarElemento(1,3,0);
  temp->ColocarElemento(2,0,0);
  temp->ColocarElemento(2,1,0);
  temp->ColocarElemento(2,2,1.0);
  temp->ColocarElemento(2,3,0);
  temp->ColocarElemento(3,0,translacao[0]);
  temp->ColocarElemento(3,1,translacao[1]);
  temp->ColocarElemento(3,2,translacao[2]);
  temp->ColocarElemento(3,3,1.0);
  return temp;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizTransformacaoRotacaoZ(float anguloGraus)
{
  Matriz* temp = new Matriz(4,4);
  double radianos = (anguloGraus/180.0) * M_PI;

  // Matriz Rotacao eixo Z
  temp->ColocarElemento(0,0,(float)cos(radianos));
  temp->ColocarElemento(0,1,(float)sin(radianos));
  temp->ColocarElemento(0,2,0);
  temp->ColocarElemento(0,3,0);
  temp->ColocarElemento(1,0,(float)-sin(radianos));
  temp->ColocarElemento(1,1,(float)cos(radianos));
  temp->ColocarElemento(1,2,0);
  temp->ColocarElemento(1,3,0);
  temp->ColocarElemento(2,0,0);
  temp->ColocarElemento(2,1,0);
  temp->ColocarElemento(2,2,1.0);
  temp->ColocarElemento(2,3,0);
  temp->ColocarElemento(3,0,0);
  temp->ColocarElemento(3,1,0);
  temp->ColocarElemento(3,2,0);
  temp->ColocarElemento(3,3,1.0);
  return temp;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizTransformacaoRotacaoY(float anguloGraus)
{
  Matriz* temp = new Matriz(4,4);
  double radianos = (anguloGraus/180.0) * M_PI;

  // Matriz Rotacao eixo Y
  temp->ColocarElemento(0,0,(float)cos(radianos));
  temp->ColocarElemento(0,1,0);
  temp->ColocarElemento(0,2,(float)-sin(radianos));
  temp->ColocarElemento(0,3,0);
  temp->ColocarElemento(1,0,0);
  temp->ColocarElemento(1,1,1);
  temp->ColocarElemento(1,2,0);
  temp->ColocarElemento(1,3,0);
  temp->ColocarElemento(2,0,(float)sin(radianos));
  temp->ColocarElemento(2,1,0);
  temp->ColocarElemento(2,2,(float)cos(radianos));
  temp->ColocarElemento(2,3,0);
  temp->ColocarElemento(3,0,0);
  temp->ColocarElemento(3,1,0);
  temp->ColocarElemento(3,2,0);
  temp->ColocarElemento(3,3,1.0);

  return temp;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizTransformacaoRotacaoX(float anguloGraus)
{
  Matriz* temp = new Matriz(4,4);
  double radianos = (anguloGraus/180.0) * M_PI;

  // Matriz Rotacao eixo X
  temp->ColocarElemento(0,0,1);
  temp->ColocarElemento(0,1,0);
  temp->ColocarElemento(0,2,0);
  temp->ColocarElemento(0,3,0);
  temp->ColocarElemento(1,0,0);
  temp->ColocarElemento(1,1,(float)cos(radianos));
  temp->ColocarElemento(1,2,(float)sin(radianos));
  temp->ColocarElemento(1,3,0);
  temp->ColocarElemento(2,0,0);
  temp->ColocarElemento(2,1,(float)-sin(radianos));
  temp->ColocarElemento(2,2,(float)cos(radianos));
  temp->ColocarElemento(2,3,0);
  temp->ColocarElemento(3,0,0);
  temp->ColocarElemento(3,1,0);
  temp->ColocarElemento(3,2,0);
  temp->ColocarElemento(3,3,1.0);

  return temp;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizTransformacaoEscalonamento(float* escalonamento)
{
  Matriz* matrizEscalonamento = new Matriz(4,4);
  matrizEscalonamento->ColocarElemento(0,0,escalonamento[0]);
  matrizEscalonamento->ColocarElemento(0,1,0);
  matrizEscalonamento->ColocarElemento(0,2,0);
  matrizEscalonamento->ColocarElemento(0,3,0);
  matrizEscalonamento->ColocarElemento(1,0,0);
  matrizEscalonamento->ColocarElemento(1,1,escalonamento[1]);
  matrizEscalonamento->ColocarElemento(1,2,0);
  matrizEscalonamento->ColocarElemento(1,3,0);
  matrizEscalonamento->ColocarElemento(2,0,0);
  matrizEscalonamento->ColocarElemento(2,1,0);
  matrizEscalonamento->ColocarElemento(2,2,escalonamento[2]);
  matrizEscalonamento->ColocarElemento(2,3,0);
  matrizEscalonamento->ColocarElemento(3,0,0);
  matrizEscalonamento->ColocarElemento(3,1,0);
  matrizEscalonamento->ColocarElemento(3,2,0);
  matrizEscalonamento->ColocarElemento(3,3,1.0);
  return matrizEscalonamento;
}
//---------------------------------------------------------------------------
Matriz* Matriz::CriarMatrizFwdDiff(int nrPassos)
{
  float   passo = 1.0/nrPassos;
  float   passo2 = passo*passo;
  float   passo3 = passo2*passo;

  // Monta a matriz
  Matriz* matrizFwdDiff = new Matriz(4,4);
  matrizFwdDiff->ColocarElemento(0,0,0.0);
  matrizFwdDiff->ColocarElemento(0,1,0.0);
  matrizFwdDiff->ColocarElemento(0,2,0.0);
  matrizFwdDiff->ColocarElemento(0,3,1.0);
  matrizFwdDiff->ColocarElemento(1,0,passo3);
  matrizFwdDiff->ColocarElemento(1,1,passo2);
  matrizFwdDiff->ColocarElemento(1,2,passo);
  matrizFwdDiff->ColocarElemento(1,3,0.0);
  matrizFwdDiff->ColocarElemento(2,0,6*passo3);
  matrizFwdDiff->ColocarElemento(2,1,2*passo2);
  matrizFwdDiff->ColocarElemento(2,2,0.0);
  matrizFwdDiff->ColocarElemento(2,3,0.0);
  matrizFwdDiff->ColocarElemento(3,0,6*passo3);
  matrizFwdDiff->ColocarElemento(3,1,0.0);
  matrizFwdDiff->ColocarElemento(3,2,0.0);
  matrizFwdDiff->ColocarElemento(3,3,0.0);
  return matrizFwdDiff;
}
//---------------------------------------------------------------------------
Matriz* Matriz::GeraTransposta()
{
  Matriz* transposta = new Matriz(nrColunas, nrLinhas);
  for (int i=0;i<nrColunas;i++){
    for(int j=0;j<nrLinhas;j++){
      float numero = GetElemento(j,i);
      transposta->ColocarElemento(i,j,numero);
    }
  }
  return transposta;
}
//---------------------------------------------------------------------------
Matriz* Matriz::AdicionaTransformacao(Matriz* uma, Matriz* transformacao, bool apaga) // Apaga as duas matrizes e retorna uma nova
{
  Matriz* temp;
  temp = Matriz::MultiplicaMatriz(uma, transformacao);
  if (apaga) {
    delete uma;
    delete transformacao;
  }
  return temp;
}
//---------------------------------------------------------------------------
void Matriz::SomaComLinhaAbaixo()
{
  for(int i=0; i<nrLinhas-1;i++){
    for(int j=0;j<nrColunas;j++){
      ColocarElemento(i,j,GetElemento(i,j) +GetElemento(i+1,j));
    }
  }
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
