//---------------------------------------------------------------------------


#pragma hdrstop

#include "ObjetoTriangularizado.h"
#include "Ponto.h"
#include "Triangulo.h"
#include "Cor.h"
#include "USisGrafico.h"
#include "Matriz.h"


//---------------------------------------------------------------------------
ObjetoTriangularizado::ObjetoTriangularizado(
  string nome,
  unsigned int _amb_ref_coef,
  unsigned int _diff_ref_coef,
  unsigned int _spec_ref_coef,
  float _shine_coef,
  vector<Triangulo*>* _triangulos
) : ObjetoGrafico(nome)
{
  amb_ref_coef = new Cor(_amb_ref_coef);
  diff_ref_coef = new Cor(_diff_ref_coef);
  spec_ref_coef = new Cor(_spec_ref_coef);
  shine_coef = _shine_coef;
  setTriangulos(_triangulos);

}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::setAmbRefCoef(Cor* amb_ref_coef)
{
  this->amb_ref_coef = amb_ref_coef;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::setDiffRefCoef(Cor* diff_ref_coef)
{
  this->diff_ref_coef = diff_ref_coef;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::setSpecRefCoef(Cor* spec_ref_coef)
{
  this->spec_ref_coef = spec_ref_coef;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::setShineCoef(float shine_coef)
{
  this->shine_coef = shine_coef;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::setTriangulos(vector<Triangulo*>* triangulos)
{
  if(triangulos){
    this->triangulos = triangulos;
    for (unsigned int i = 0; i < triangulos->size(); i++)
      (*triangulos)[i]->SetPai(this);
  }
}
//---------------------------------------------------------------------------
Cor* ObjetoTriangularizado::getAmbRefCoef()
{
  return amb_ref_coef;
}
//---------------------------------------------------------------------------
Cor* ObjetoTriangularizado::getDiffRefCoef()
{
  return diff_ref_coef;
}
//---------------------------------------------------------------------------
Cor* ObjetoTriangularizado::getSpecRefCoef()
{
  return spec_ref_coef;
}
//---------------------------------------------------------------------------
float ObjetoTriangularizado::getShineCoef()
{
  return shine_coef;
}
//---------------------------------------------------------------------------
Ponto* ObjetoTriangularizado::Centro()
{
  int numTriangulos = triangulos->size();
  float somaX = 0;
  float somaY = 0;
  float somaZ = 0;
  for (unsigned int i = 0; i < triangulos->size(); i++)
  {
    Triangulo* trianguloAtual = (*triangulos)[i];
    somaX += trianguloAtual->GetP1()->GetX();
    somaX += trianguloAtual->GetP2()->GetX();
    somaX += trianguloAtual->GetP3()->GetX();

    somaY += trianguloAtual->GetP1()->GetY();
    somaY += trianguloAtual->GetP2()->GetY();
    somaY += trianguloAtual->GetP3()->GetY();

    somaZ += trianguloAtual->GetP1()->GetZ();
    somaZ += trianguloAtual->GetP2()->GetZ();
    somaZ += trianguloAtual->GetP3()->GetZ();
  }
  return new Ponto("centro", somaX/(numTriangulos*3), somaY/(numTriangulos*3), somaZ/(numTriangulos*3));
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao, bool CPP)
{
  Ponto* centro = NULL;
  if (!CPP)
    centro = Centro();
  Matriz* matriz = CriarMatrizTransformacoes(translacao,escalonamento,rotacao,centro);
  for (unsigned int i = 0; i < triangulos->size(); i++)
    (*triangulos)[i]->AplicaTransformacao(matriz, CPP);
  if(rotacao)
    RotacionaVetores(rotacao[0],rotacao[1],rotacao[2]);

  delete matriz;
  delete centro;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  for (unsigned int i = 0; i < triangulos->size(); i++)
    (*triangulos)[i]->AplicaTransformacao(matrizDeTransformacao, CPP);
}
//------------------------------------------------------------------------------
void ObjetoTriangularizado::AplicaTransformacaoNosVetores(Matriz* matrizDeTransformacao, bool CPP)
{
  for (unsigned int i = 0; i < triangulos->size(); i++)
    (*triangulos)[i]->AplicaTransformacaoNoVetor(matrizDeTransformacao, CPP);
}
//------------------------------------------------------------------------------
void ObjetoTriangularizado::RotacionaVetores(float x, float y, float z)
{
  Matriz* matrizRotacao = Matriz::CriarMatrizTransformacaoRotacaoX(x);
  Matriz* temp = Matriz::CriarMatrizTransformacaoRotacaoY(y);
  matrizRotacao = Matriz::AdicionaTransformacao(matrizRotacao, temp, true);
  temp = Matriz::CriarMatrizTransformacaoRotacaoZ(z);
  matrizRotacao = Matriz::AdicionaTransformacao(matrizRotacao, temp, true);

  for (unsigned int i = 0; i < triangulos->size(); i++)
    (*triangulos)[i]->AplicaTransformacaoNoVetor(matrizRotacao, false);

  delete matrizRotacao;
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::FazTransformadaDeViewPort(SistemaGrafico* sg)
{
  for (unsigned int i = 0; i < triangulos->size(); i++)
    (*triangulos)[i]->FazTransformadaDeViewPort(sg);
}
//---------------------------------------------------------------------------
void ObjetoTriangularizado::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);

  output <<  amb_ref_coef->RBGParaNumero() << " ";
  output <<  diff_ref_coef->RBGParaNumero() << " ";
  output <<  spec_ref_coef->RBGParaNumero() << " ";
  output <<  shine_coef << " ";

  output << triangulos->size()<< " ";
  for (unsigned int i=0;i<triangulos->size();i++)
    (*triangulos)[i]->SalvaNoStream(output);

}
//---------------------------------------------------------------------------
ObjetoTriangularizado* ObjetoTriangularizado::LeDoStream(istream &input, int versao)
{
  string nome = ObjetoGrafico::LeNomeDoStream(input);

  int amb_ref_coef, diff_ref_coef, spec_ref_coef;
  float shine_coef;

  input >>  amb_ref_coef;
  input >>  diff_ref_coef;
  input >>  spec_ref_coef;
  input >>  shine_coef;

  int nrPontos;
  input >> nrPontos;
  vector<Triangulo*>* triang = new vector<Triangulo*>;
  for(int i=0 ;i<nrPontos;i++)
    triang->push_back(Triangulo::LeDoStream(input,versao));

  return new ObjetoTriangularizado(nome,amb_ref_coef, diff_ref_coef, spec_ref_coef, shine_coef, triang );
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
 