//---------------------------------------------------------------------------
#pragma hdrstop
#include "Ponto.h"
#include "USisGrafico.h"
#include "Window.h"

//---------------------------------------------------------------------------
Ponto::Ponto(string nome, float _x, float _y, float _z, float _w)
 : ObjetoGrafico(nome),
   x(_x),
   y(_y),
   z(_z),
   w(_w),
   xWin(_x),
   yWin(_y),
   zWin(_z),
   wWin(_w),
   xViewPort(0),
   yViewPort(0),
   pontoLuz(false)
{
}
//---------------------------------------------------------------------------
Ponto::Ponto(const Ponto& outro) :
   ObjetoGrafico(outro.nome),
   x(outro.x),
   y(outro.y),
   z(outro.z),
   w(outro.w),
   xWin(outro.xWin),
   yWin(outro.yWin),
   zWin(outro.zWin),
   wWin(outro.wWin),
   pontoLuz(outro.pontoLuz)
{
}
//---------------------------------------------------------------------------
Ponto::Ponto() :
 ObjetoGrafico("Ponto"),
   x(0.0),
   y(0.0),
   z(0.0),
   w(1.0),
   xWin(0.0),
   yWin(0.0),
   zWin(0.0),
   wWin(0.0),
   pontoLuz(false)
{
}
//---------------------------------------------------------------------------
Ponto* Ponto::Centro()
{
  return new Ponto(*this);
}
//---------------------------------------------------------------------------
void Ponto::SetX(float valor)
{
  x = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetY(float valor)
{
  y = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetZ(float valor)
{
  z = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetW(float valor)
{
  w = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetXWindow(float valor)
{
  xWin = valor;
}

//---------------------------------------------------------------------------
void Ponto::SetYWindow(float valor)
{
  yWin = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetZWindow(float valor)
{
  zWin = valor;
}
//---------------------------------------------------------------------------
void Ponto::SetWWindow(float valor)
{
  wWin = valor;
}
//---------------------------------------------------------------------------
bool Ponto::GetPontoLuz()
{
  return pontoLuz;
}
//---------------------------------------------------------------------------
void Ponto::SetPontoLuz(bool _ponto)
{
  pontoLuz = _ponto;
}
//---------------------------------------------------------------------------
void Ponto::FazTransformadaDeViewPort(SistemaGrafico* sg)
{
  float pos_x = xWin;
  float pos_y = yWin;
  if(!sg->ProjecaoParalela()){
    float distanciaCOP = sg->GetWindow()->GetDistanciaCOP();
    if (!(zWin / distanciaCOP)==0) {
      pos_x = pos_x / (zWin / distanciaCOP);
      pos_y = pos_y / (zWin / distanciaCOP);
    } else {
      pos_x = 0;
      pos_y = 0;
    }
  }

  xViewPort = sg->PegaX(pos_x);
  yViewPort = sg->PegaY(pos_y);
}
//---------------------------------------------------------------------------
void Ponto::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao,bool CPP)
{
    Ponto* centro = NULL;
    if (!CPP)
      centro = Centro();
    Matriz* matriz = CriarMatrizTransformacoes(translacao,escalonamento,rotacao,centro);
    AplicaTransformacaoNoPonto(this, matriz,CPP);
    delete matriz;
    delete centro;
}
//---------------------------------------------------------------------------
void Ponto::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  AplicaTransformacaoNoPonto(this, matrizDeTransformacao,CPP);
}
//---------------------------------------------------------------------------
void Ponto::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);
  output << handle << " " << x << " " << y << " " << z << " " << pontoLuz << " ";
}
//---------------------------------------------------------------------------
Ponto* Ponto::LeDoStream(istream &input, int versao)
{
  float x, y, z;
  unsigned int ponteiro;
  bool pontoLuz;
  string nome = ObjetoGrafico::LeNomeDoStream(input);
  input >> ponteiro >> x >> y >> z;
  if (versao < VERSAO_19062004)
    pontoLuz = false;
  else
    input >> pontoLuz;
  Ponto* p = new Ponto(nome,x,y,z);
  p->SetPontoLuz(pontoLuz);
  p->SetHandle(ponteiro);
  return p;
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
