#pragma hdrstop

#include "Superficies.h"
#include "Ponto.h"
#include "Matriz.h"
#include "Curva.h"

//---------------------------------------------------------------------------

Superficie::Superficie(string nome, MatrizPontos* _matrizPontos, tipoCurva _tipo)
 : ObjetoGrafico(nome),
   matrizPontos(_matrizPontos),
   tipo(_tipo)
{
  xDim = matrizPontos->size();
  yDim = (*matrizPontos)[0].size();
  matrizMGMTX = GeraMatrizMGMT(EIXO_X);
  matrizMGMTY = GeraMatrizMGMT(EIXO_Y);
  matrizMGMTZ = GeraMatrizMGMT(EIXO_Z);

}
//---------------------------------------------------------------------------
Superficie::~Superficie()
{
  for (unsigned int i=0;i<xDim;i++)
    for (unsigned int j=0;j<yDim;j++)
      delete (*matrizPontos)[i][j];
  delete matrizPontos;

  ApagaMatrizes();
}
//---------------------------------------------------------------------------
vector<Matriz*>* Superficie::GeraMatrizMGMT(Eixo eixo)
{
  vector<Matriz*>* retorno = new vector<Matriz*>;
  vector<MatrizPontos*> superficies;
  int incremento = 1;
  if (tipo == BEZIER)
    incremento = 3;
  for (unsigned int i=0; i<(xDim-3); i = i+incremento) {
    for (unsigned int j=0; j<(yDim-3); j = j+incremento) {
      MatrizPontos* m = new MatrizPontos;
      m->resize(4);
      for(int x = 0; x < 4; x++)
        (*m)[x].resize(4);
      for (unsigned int k=0; k<4; k++)
        for (unsigned int l=0; l<4; l++)
          (*m)[k][l] = (*matrizPontos)[k+i][l+j];
      superficies.push_back(m);
    }
  }

  Matriz* matrizM = Curva::MatrizDaCurva(tipo);
  Matriz* matrizMT = matrizM->GeraTransposta();
  for (unsigned int t=0; t<superficies.size(); t++) {
    MatrizPontos* atual = superficies[t];
    Matriz* matrizG = new Matriz(4, 4);

    switch(eixo){
      case EIXO_X:
      for (unsigned int i=0;i<4;i++)
        for (unsigned int j=0;j<4;j++)
          matrizG->ColocarElemento(j,i,(*atual)[i][j]->GetXWindow());
      break;
      case EIXO_Y:
      for (unsigned int i=0;i<4;i++)
        for (unsigned int j=0;j<4;j++)
          matrizG->ColocarElemento(j,i,(*atual)[i][j]->GetYWindow());
      break;
      case EIXO_Z:
      for (unsigned int i=0;i<4;i++)
        for (unsigned int j=0;j<4;j++)
          matrizG->ColocarElemento(j,i,(*atual)[i][j]->GetZWindow());
      break;
    }

    Matriz* temp = Matriz::MultiplicaMatriz(matrizM, matrizG);
    Matriz* ret = Matriz::MultiplicaMatriz(temp, matrizMT);

    delete matrizG;
    delete temp;
    retorno->push_back(ret);
  }

  delete matrizM;
  delete matrizMT;

  for(unsigned int i=0;i<superficies.size();i++)
    delete superficies[i];

  return retorno;
}
//---------------------------------------------------------------------------
vector<Matriz*>* Superficie::GetMatrizMGMT(Eixo eixo)
{
  switch(eixo){
    case EIXO_X:
      return matrizMGMTX;
    case EIXO_Y:
      return matrizMGMTY;
    case EIXO_Z:
      return matrizMGMTZ;
  }
  return NULL;
}
//---------------------------------------------------------------------------
Ponto* Superficie::GeraPonto(float s, float t)
{
/*
  Matriz matrizS(1,4);
  Matriz matrizT(4,1);
  float t3, t2;
  float s3, s2;
  t2 = t*t;
  t3 = t2*t;
  s2 = s*s;
  s3 = s2*s;
  matrizS.ColocarElemento(0,0,s3);
  matrizS.ColocarElemento(0,1,s2);
  matrizS.ColocarElemento(0,2,s);
  matrizS.ColocarElemento(0,3,1);

  matrizT.ColocarElemento(0,0,t3);
  matrizT.ColocarElemento(1,0,t2);
  matrizT.ColocarElemento(2,0,t);
  matrizT.ColocarElemento(3,0,1);

  Matriz* temp = Matriz::MultiplicaMatriz(&matrizS,matrizMGMTX);
  Matriz* matrizX = Matriz::MultiplicaMatriz(temp,&matrizT);
  delete temp;

  temp = Matriz::MultiplicaMatriz(&matrizS,matrizMGMTY);
  Matriz* matrizY = Matriz::MultiplicaMatriz(temp,&matrizT);
  delete temp;

  temp = Matriz::MultiplicaMatriz(&matrizS,matrizMGMTZ);
  Matriz* matrizZ = Matriz::MultiplicaMatriz(temp,&matrizT);
  delete temp;

  float valorX = matrizX->GetElemento(0,0);
  float valorY = matrizY->GetElemento(0,0);
  float valorZ = matrizZ->GetElemento(0,0);

  Ponto* retorno = new Ponto("",valorX,valorY,valorZ);
  delete matrizX;
  delete matrizY;
  delete matrizZ;

  return retorno;
  */
  return NULL;
}
//---------------------------------------------------------------------------
void Superficie::ApagaMatrizes()
{
  for (unsigned int i=0;i<matrizMGMTX->size();i++){
    delete (*matrizMGMTX)[i];
    delete (*matrizMGMTY)[i];
    delete (*matrizMGMTZ)[i];
  }
  delete matrizMGMTX;
  delete matrizMGMTY;
  delete matrizMGMTZ;
}
//---------------------------------------------------------------------------
Ponto* Superficie::Centro()
{
  float dx = 0.0;
  float dy = 0.0;
  float dz = 0.0;
  for (unsigned int i=0;i<xDim;i++){
    for (unsigned int j=0;j<yDim;j++) {
      dx += (*matrizPontos)[i][j]->GetX();
      dy += (*matrizPontos)[i][j]->GetY();
      dz += (*matrizPontos)[i][j]->GetZ();
    }
  }
  return new Ponto("Centro",dx/(yDim*xDim),dy/(yDim*xDim),dz/(yDim*xDim));
}

//---------------------------------------------------------------------------
void Superficie::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao, bool CPP)
{
  Ponto* centro = NULL;
  if (!CPP)
    centro = Centro();
  Matriz* matriz = CriarMatrizTransformacoes(translacao,escalonamento,rotacao,centro);
  for (unsigned int i=0;i<xDim;i++){
    for (unsigned int j=0;j<yDim;j++) {
      Ponto* p = (*matrizPontos)[i][j];
      AplicaTransformacaoNoPonto(p, matriz,CPP);
    }
  }
  ApagaMatrizes();
  matrizMGMTX = GeraMatrizMGMT(EIXO_X);
  matrizMGMTY = GeraMatrizMGMT(EIXO_Y);
  matrizMGMTZ = GeraMatrizMGMT(EIXO_Z);

  delete matriz;
  delete centro;
}
//---------------------------------------------------------------------------
void Superficie::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  for (unsigned int i=0;i<xDim;i++){
    for (unsigned int j=0;j<yDim;j++) {
      Ponto* p = (*matrizPontos)[i][j];
      AplicaTransformacaoNoPonto(p, matrizDeTransformacao,CPP);
    }
  }

  ApagaMatrizes();
  matrizMGMTX = GeraMatrizMGMT(EIXO_X);
  matrizMGMTY = GeraMatrizMGMT(EIXO_Y);
  matrizMGMTZ = GeraMatrizMGMT(EIXO_Z);
}
//---------------------------------------------------------------------------
void Superficie::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);
  output << tipo << " ";
  output << xDim<< " ";
  output << yDim<< " ";
  for (unsigned int i=0;i<xDim;i++)
    for (unsigned int j=0;j<yDim;j++)
     (*matrizPontos)[i][j]->SalvaNoStream(output);
}
//---------------------------------------------------------------------------
Superficie* Superficie::LeDoStream(istream &input, int versao)
{
  string nome = ObjetoGrafico::LeNomeDoStream(input);
  int tipo;
  input >> tipo;
  int tamanhoX, tamanhoY;
  input >> tamanhoX;
  input >> tamanhoY;

  MatrizPontos* matrizDePontos = new MatrizPontos;
  matrizDePontos->resize(tamanhoX);
  for(int x = 0; x < tamanhoX; x++)
    (*matrizDePontos)[x].resize(tamanhoY);

  for (int i=0;i<tamanhoX;i++)
    for (int j=0;j<tamanhoY;j++)
      (*matrizDePontos)[i][j] = Ponto::LeDoStream(input, versao);

  Superficie* sup = new Superficie(nome,matrizDePontos, (tipoCurva)tipo);

  return sup;
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
