//---------------------------------------------------------------------------
#pragma hdrstop

#include "Triangulo.h"
#include "Ponto.h"
#include "Reta.h"
#include "USisGrafico.h"

Triangulo::Triangulo(string nome, Ponto* p1, Ponto* p2, Ponto* p3, Ponto* _vetor)
:WireFrame(nome)
{
  AdicionaPonto(p1);
  AdicionaPonto(p2);
  AdicionaPonto(p3);
  AdicionaAresta(new Reta("",p1, p2));
  AdicionaAresta(new Reta("",p2, p3));
  AdicionaAresta(new Reta("",p3, p1));
  vetor = _vetor;
  vetorSimples = new PontoSimples(vetor->GetX(),vetor->GetY(),vetor->GetZ());
}
//---------------------------------------------------------------------------
Ponto* Triangulo::GetP1()
{
  return pontos[0];
}
//---------------------------------------------------------------------------
Ponto* Triangulo::GetP2()
{
  return pontos[1];
}
//---------------------------------------------------------------------------
Ponto* Triangulo::GetP3()
{
  return pontos[2];
}
//---------------------------------------------------------------------------
Ponto* Triangulo::PontoComMenorY() const
{
  Ponto* pontoMenorY = pontos[0];
  float menorY = pontos[0]->GetYViewPort();
  for(int i=1;i<3;i++){
    if (pontos[i]->GetYViewPort() < menorY){
      pontoMenorY = pontos[i];
      menorY =pontos[i]->GetYViewPort();
    } else if(pontos[i]->GetYViewPort() == menorY){
      if (pontos[i]->GetXViewPort() < pontoMenorY->GetXViewPort()){
        pontoMenorY = pontos[i];
        menorY =pontos[i]->GetYViewPort();
      }
    }
  }
  return pontoMenorY;
}
//---------------------------------------------------------------------------
Ponto* Triangulo::PontoComMaiorY() const
{
  Ponto* pontoMaiorY = pontos[0];
  float maiorY = pontos[0]->GetYViewPort();
  for(int i=1;i<3;i++){
   if (pontos[i]->GetYViewPort() > maiorY){
      pontoMaiorY = pontos[i];
      maiorY =pontos[i]->GetYViewPort();
    }
  }
  return pontoMaiorY;
}
//---------------------------------------------------------------------------
Ponto* Triangulo::PontoComYIntermediario() const
{
  Ponto* pontoMaiorY = PontoComMaiorY();
  Ponto* pontoMenorY = PontoComMenorY();
  Ponto* retorno;

  for(int i=0;i<3;i++)
   if ((pontos[i] != pontoMaiorY) && (pontos[i] != pontoMenorY))
      retorno = pontos[i];
  return retorno;
}
//---------------------------------------------------------------------------
void Triangulo::FazTransformadaDeViewPort(SistemaGrafico* sg)
{
  GetP1()->FazTransformadaDeViewPort(sg);
  GetP2()->FazTransformadaDeViewPort(sg);
  GetP3()->FazTransformadaDeViewPort(sg);
}
//---------------------------------------------------------------------------
void Triangulo::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);
  pontos[0]->SalvaNoStream(output);
  pontos[1]->SalvaNoStream(output);
  pontos[2]->SalvaNoStream(output);
  vetor->SalvaNoStream(output);
}
//---------------------------------------------------------------------------
Triangulo* Triangulo::LeDoStream(istream &input, int versao)
{
  string nome = ObjetoGrafico::LeNomeDoStream(input);
  Ponto* p1 = Ponto::LeDoStream(input, versao);
  Ponto* p2 = Ponto::LeDoStream(input, versao);
  Ponto* p3 = Ponto::LeDoStream(input, versao);
  Ponto* vetor = Ponto::LeDoStream(input, versao);
  return new Triangulo(nome, p1, p2, p3, vetor);
}
//---------------------------------------------------------------------------
Ponto* Triangulo::GetVetor()
{
  return vetor;
}
//---------------------------------------------------------------------------
void Triangulo::AplicaTransformacaoNoVetor(Matriz* matrizDeTransformacao, bool CPP)
{
  vetor->AplicaTransformacao(matrizDeTransformacao ,CPP);
  delete vetorSimples;
  vetorSimples = new PontoSimples(vetor->GetX(),vetor->GetY(),vetor->GetZ());
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
