#pragma hdrstop

#include <Math>
#include <Windows.h>

#include "USisGrafico.h"
#include "ObjetoGrafico.h"
#include "ObjetoTriangularizado.h"
#include "Reta.h"
#include "Ponto.h"
#include "WireFrame.h"
#include "Window.h"
#include "Matriz.h"
#include "Curva.h"
#include "Superficies.h"
#include "Triangulo.h"
#include "Cor.h"

//---------------------------------------------------------------------------
SistemaGrafico::SistemaGrafico(TCanvas* _canvas) :
  pi_180(M_PI/180),
  _180_pi(180/M_PI)
{
  canvasReal = _canvas;
  bitmapTemp = new Graphics::TBitmap();
  bitmapTemp->Height = TAM_TOTAL_VIEWPORT;
  bitmapTemp->Width = TAM_TOTAL_VIEWPORT;
  canvas = bitmapTemp->Canvas;
  window = new Window(0,1000,0,1000);
  clipping = false;
  projecaoParalela = true;
  fatorK = 0.01;
  componenteAmbiente = true;
  componenteDifusa = true;
  componenteEspecular = true;
  usarSombra = true;

  DesenhaTudo();
}
//---------------------------------------------------------------------------
SistemaGrafico::~SistemaGrafico()
{
  for(unsigned int i=0;i<listaObjetos.size();i++)
    delete listaObjetos[i];
  delete window;
  delete bitmapTemp;
}
//---------------------------------------------------------------------------
Window* SistemaGrafico::GetWindow()
{
  return window;
}
//---------------------------------------------------------------------------
bool SistemaGrafico::ProjecaoParalela()
{
  return projecaoParalela;
}
//---------------------------------------------------------------------------
void SistemaGrafico::InsereReta(Reta* reta)
{
  listaObjetos.push_back(reta);
  RotacionaWindowEixoZ(0.0);
}
//---------------------------------------------------------------------------
void SistemaGrafico::InserePonto(Ponto* ponto)
{
  listaObjetos.push_back(ponto);
  RotacionaWindowEixoZ(0.0);
}
//---------------------------------------------------------------------------
void SistemaGrafico::InsereWireFrame(WireFrame* wireframe)
{
  listaObjetos.push_back(wireframe);
  RotacionaWindowEixoZ(0.0);
}
//---------------------------------------------------------------------------

void SistemaGrafico::ZoomIn(int passo)// Diminui o tamanho do window
{
  window->ZoomIn(passo);
  DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::ZoomOut(int passo) // Aumenta o tamanho do window
{
  window->ZoomOut(passo);
  DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::Mover(int passo, int d){ // Desloca o window

  window->Mover(passo,d);
  Projecao(0.0,0.0,0.0);
}
//---------------------------------------------------------------------------
int SistemaGrafico::PegaX(float XW) // Faz a transformada de ViewPort no X
{
  int XVPMax = TAM_CANVAS;
  float xWMin = window->GetXMinWindow();
  float xWMax = window->GetXMaxWindow();
  float divisao = (xWMax - xWMin) == 0 ? 0 :(XW - xWMin)/(xWMax - xWMin);
  int x = divisao*XVPMax;
  return x+TAM_MARGEM_CLIPPING;
}
//---------------------------------------------------------------------------
int SistemaGrafico::PegaY(float YW) // Faz a transformada de ViewPort no Y
{
  int YVPMax = TAM_CANVAS;
  float yWMin = window->GetYMinWindow();
  float yWMax = window->GetYMaxWindow();
  float divisao = (yWMax - yWMin) == 0 ? 0 : (YW - yWMin)/(yWMax - yWMin);
  int y = (1-divisao)*YVPMax;
  return y+TAM_MARGEM_CLIPPING;
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaTudo()
{
  if (rayTracing){
    ScanConvertion();
    DesenhaRayTracing();
    return;
  }
  // Limpa a tela
  canvas->Brush->Color = clWhite;
  canvas->Brush->Style = bsSolid;
  int posicaoFinal = TAM_CANVAS + TAM_MARGEM_CLIPPING;
  canvas->FillRect(canvas->ClipRect);

  // Percorre a lista de objetos grficos e desenha da tela
  for(unsigned int i=0;i<listaObjetos.size();i++){
    ObjetoGrafico* atual = listaObjetos[i];
    canvas->Pen->Color = (TColor)atual->GetCor().RBGParaNumero();
    Ponto* ponto = dynamic_cast<Ponto*> (atual);
    if (ponto) {
      float pos_x = ponto->GetXWindow();
      float pos_y = ponto->GetYWindow();
      if(!projecaoParalela) {
        float pos_z = ponto->GetZWindow();
        float distanciaCOP = window->GetDistanciaCOP();
        try {
          pos_x = pos_x / (pos_z / distanciaCOP);
          pos_y = pos_y / (pos_z / distanciaCOP);
        } catch(...){
          pos_x=0;
          pos_y=0;
        }// Ignorar divises por 0
      }
      bool desenhar = true;
      if (clipping)
        desenhar = ClippingDePonto(pos_x,pos_y);
      if (desenhar)
        canvas->Pixels[PegaX(pos_x)][PegaY(pos_y)] = clBlack;
    }
    Reta* reta = dynamic_cast<Reta*> (atual);
    if (reta) {
      float pos_x1 = reta->GetP1().GetXWindow();
      float pos_x2 = reta->GetP2().GetXWindow();
      float pos_y1 = reta->GetP1().GetYWindow();
      float pos_y2 = reta->GetP2().GetYWindow();
      float pos_z1 = reta->GetP1().GetZWindow();
      float pos_z2 = reta->GetP2().GetZWindow();
      DesenhaLinha(pos_x1,pos_y1,pos_z1, pos_x2,pos_y2,pos_z2);
    }
    WireFrame* wireframe = dynamic_cast<WireFrame*>(atual);
    if(wireframe)
      DesenhaWireFrame(wireframe);
    Curva* curva = dynamic_cast<Curva*>(atual);
    if (curva)
      DesenhaCurva(curva);
    Superficie* superficie = dynamic_cast<Superficie*>(atual);
    if (superficie)
      DesenhaSuperficieFwdDif(superficie);
    ObjetoTriangularizado* objTriang = dynamic_cast<ObjetoTriangularizado*>(atual);
    if (objTriang)
      DesenhaObjetoTriangularizado(objTriang);

  }
  // Desenha retngulo vermelho
  canvas->Brush->Color = clRed;
  canvas->FrameRect(
     TRect(TAM_MARGEM_CLIPPING,TAM_MARGEM_CLIPPING,posicaoFinal+1,posicaoFinal+1)
  );

  canvasReal->Draw(0,0,bitmapTemp);
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaWireFrame(WireFrame* wireframe)
{
  vector<Reta*>* arestas = wireframe->RetornaArestas();
  for(unsigned int i=0;i<arestas->size();i++){
    Reta* reta = (*arestas)[i];
    float pos_x1 = reta->GetP1().GetXWindow();
    float pos_x2 = reta->GetP2().GetXWindow();
    float pos_y1 = reta->GetP1().GetYWindow();
    float pos_y2 = reta->GetP2().GetYWindow();
    float pos_z1 = reta->GetP1().GetZWindow();
    float pos_z2 = reta->GetP2().GetZWindow();
    DesenhaLinha(pos_x1,pos_y1,pos_z1, pos_x2,pos_y2,pos_z2);
  }
}
//---------------------------------------------------------------------------
void SistemaGrafico::AplicaTransformacao(
  float* translacao,
  float* escalonamento,
  float* rotacao,
  int posicao
)
{
  listaObjetos[posicao]->AplicaTransformacao(translacao, escalonamento, rotacao);
  RotacionaWindowEixoZ(0.0);
}
//---------------------------------------------------------------------------
void SistemaGrafico::RotacionaWindowEixoX(float valor)
{
  window->SetAnguloY(window->GetAnguloY()+valor);
  Projecao(valor,0.0,0.0);
}
//---------------------------------------------------------------------------
void SistemaGrafico::RotacionaWindowEixoY(float valor)
{
  window->SetAnguloX(window->GetAnguloX()+valor);
  Projecao(0.0,valor,0.0);
}
//---------------------------------------------------------------------------
void SistemaGrafico::RotacionaWindowEixoZ(float valor)
{
  window->SetAngulo(window->GetAngulo()+valor);
  Projecao(0.0,0.0,valor);
}
//---------------------------------------------------------------------------
void SistemaGrafico::RayTracing(bool valor)
{
  rayTracing = valor;
}
//---------------------------------------------------------------------------
void SistemaGrafico::Projecao(float valorX,float valorY,float valorZ)
{
  Ponto* p;
  if (projecaoParalela)
    p = window->Centro();
  else
    p = &window->GetCOP();

  // Objetos
  float translada[3];
  translada[0] = -p->GetX();
  translada[1] = -p->GetY();
  translada[2] = -p->GetZ();

  Matriz* matrizTransladaCentro =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
  Ponto* pontoCentro = window->Centro();
  float transladaCentro[3];
  transladaCentro[0] = -pontoCentro->GetX();
  transladaCentro[1] = -pontoCentro->GetY();
  transladaCentro[2] = -pontoCentro->GetZ();

  Matriz* rotacaoObjetosX = Matriz::CriarMatrizTransformacaoRotacaoX(-(window->GetAnguloY() - 90));
  Matriz* rotacaoObjetosY = Matriz::CriarMatrizTransformacaoRotacaoY(-(window->GetAnguloX() - 90));
  Matriz* rotacaoObjetosZ =  Matriz::CriarMatrizTransformacaoRotacaoZ(-window->GetAngulo());

  Matriz* temp1 = Matriz::AdicionaTransformacao(matrizTransladaCentro, rotacaoObjetosX, false);
  Matriz* temp2 = Matriz::AdicionaTransformacao(temp1, rotacaoObjetosY, false);
  Matriz* projecao = Matriz::AdicionaTransformacao(temp2, rotacaoObjetosZ, false);

  delete temp1;
  delete temp2;

  // Window
  Matriz* rotacionaWindow= NULL;
  Matriz* rotacionaVetores= NULL;
  Matriz* temp;
  if (valorX != 0.0){
    temp = Matriz::AdicionaTransformacao(matrizTransladaCentro, rotacaoObjetosY, false);
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosZ, false);
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosY, rotacaoObjetosZ, true);
    delete rotacaoObjetosX;
  }else if (valorY != 0.0){
    temp = Matriz::AdicionaTransformacao(matrizTransladaCentro, rotacaoObjetosX, false);
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosZ, false);
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosX, rotacaoObjetosZ, true);
    delete rotacaoObjetosY;
  }else if (valorZ != 0.0){
    temp = Matriz::AdicionaTransformacao(matrizTransladaCentro, rotacaoObjetosX, false);
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosY, false);
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosX, rotacaoObjetosY, true);
    delete rotacaoObjetosZ;
  }

  if (!rotacionaWindow) {
    delete matrizTransladaCentro;
    delete rotacaoObjetosX;
    delete rotacaoObjetosY;
    delete rotacaoObjetosZ;
  } else {
    delete temp;
    delete matrizTransladaCentro;
  }


  if (valorX != 0.0){
    Matriz* rotacaoWindowX = Matriz::CriarMatrizTransformacaoRotacaoX(valorX);
    rotacionaWindow = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoWindowX, true);
  } else if (valorY != 0.0) {
    Matriz* rotacaoWindowY = Matriz::CriarMatrizTransformacaoRotacaoY(valorY);
    rotacionaWindow = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoWindowY, true);
  } else if (valorZ != 0.0) {
    Matriz* rotacaoWindowZ = Matriz::CriarMatrizTransformacaoRotacaoZ(valorZ);
    rotacionaWindow = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoWindowZ, true);
  }

  for(unsigned int i=0;i<listaObjetos.size();i++){
    ObjetoGrafico* atual = listaObjetos[i];
    ObjetoTriangularizado* objTriang = dynamic_cast<ObjetoTriangularizado*>(atual);
    if(objTriang && rotacionaVetores){
      objTriang->AplicaTransformacaoNosVetores(rotacionaVetores, false);
      objTriang->RotacionaVetores(-valorX, -valorY, -valorZ);
    }
  }
  delete rotacionaVetores;

  translada[0] = -translada[0];
  translada[1] = -translada[1];
  translada[2] = -translada[2];

  Matriz* rotacaoObjetosXVolta, *rotacaoObjetosYVolta, *rotacaoObjetosZVolta;
  if (rotacionaWindow) {
    rotacaoObjetosXVolta = Matriz::CriarMatrizTransformacaoRotacaoX((window->GetAnguloY() - 90));
    rotacaoObjetosYVolta = Matriz::CriarMatrizTransformacaoRotacaoY((window->GetAnguloX() - 90));
    rotacaoObjetosZVolta =  Matriz::CriarMatrizTransformacaoRotacaoZ(window->GetAngulo());
  }
  temp = NULL;
  if (valorX != 0.0){
    temp = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoObjetosZVolta, false);
    delete rotacionaWindow;
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosYVolta, false);
    delete temp;
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosZVolta,rotacaoObjetosYVolta , true);
    delete rotacaoObjetosXVolta;
  }else if (valorY != 0.0){
    temp = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoObjetosZVolta, false);
    delete rotacionaWindow;
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosXVolta, false);
    delete temp;
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosXVolta, rotacaoObjetosZVolta, true);
    delete rotacaoObjetosYVolta;
  }else if (valorZ != 0.0){
    temp = Matriz::AdicionaTransformacao(rotacionaWindow, rotacaoObjetosYVolta, false);
    delete rotacionaWindow;
    rotacionaWindow = Matriz::AdicionaTransformacao(temp, rotacaoObjetosXVolta, false);
    delete temp;
    rotacionaVetores = Matriz::AdicionaTransformacao(rotacaoObjetosXVolta, rotacaoObjetosYVolta, true);
    delete rotacaoObjetosZVolta;
  }
  if (rotacionaWindow) {
    Matriz* matrizTransladaDeVolta =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
    rotacionaWindow = Matriz::AdicionaTransformacao(rotacionaWindow, matrizTransladaDeVolta, true);
    window->AplicaTransformacao(rotacionaWindow, false);
  }

  for(unsigned int i=0;i<listaObjetos.size();i++){
    ObjetoGrafico* atual = listaObjetos[i];
    atual->AplicaTransformacao(projecao, true);
    ObjetoTriangularizado* objTriang = dynamic_cast<ObjetoTriangularizado*>(atual);
    if(objTriang && rotacionaVetores)
      objTriang->AplicaTransformacaoNosVetores(rotacionaVetores, false);
  }
  window->AplicaTransformacao(projecao, true);

  // Apaga ponteiros usados
  delete projecao;
  if (rotacionaWindow)
    delete rotacionaWindow;
  if (projecaoParalela)
    delete p;

  DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::AtivaAnimacao(int pos)
{
  listaObjetos[pos]->Animar();
}
//---------------------------------------------------------------------------
void SistemaGrafico::Animacao()
{
  unsigned int contador = 0;
  float* rotacao = new float[3];
  rotacao[0] = 0.0;
  rotacao[1] = 5.0;
  rotacao[2] = 0.0;
  for(unsigned int i=0;i<listaObjetos.size();i++){
    ObjetoGrafico* atual = listaObjetos[i];
    if(atual->Animado()){
      AplicaTransformacao(NULL,NULL,rotacao,i);
      contador++;
    }
  }
  if (contador)
    DesenhaTudo();
}
//---------------------------------------------------------------------------
bool SistemaGrafico::ClippingDePonto(float x,float y) // true se ponto deve ser desenhado
{
  if (x> window->GetXMaxWindow() || x<window->GetXMinWindow() ||
        y> window->GetYMaxWindow() || y<window->GetYMinWindow())
        return false;
  else
        return true;
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaLinha(float x0,float y0, float z0, float x1,float y1, float z1) // Coordenadas de ViewPort
{
  float pos_x1 = x0;
  float pos_y1 = y0;
  float pos_x2 = x1;
  float pos_y2 = y1;

  if(!projecaoParalela) {
    if (z0 <= 0 && z1 <= 0)
      return;
    if (z0 <=0) z0 = 0.1;
    if (z1 <=0) z1 = 0.1;
    float pos_z1 = z0;
    float pos_z2 = z1;
    float distanciaCOP = window->GetDistanciaCOP();
    try {
      pos_x1 = pos_x1 / (pos_z1 / distanciaCOP);
      pos_x2 = pos_x2 / (pos_z2 / distanciaCOP);
      pos_y1 = pos_y1 / (pos_z1 / distanciaCOP);
      pos_y2 = pos_y2 / (pos_z2 / distanciaCOP);
    } catch(...){
      pos_x1=0;
      pos_x2=0;
      pos_y1=0;
      pos_y2=0;
    }// Ignorar divises por 0
  }
  if (clipping){
    RecorteLinhaCohenSutherland(
      pos_x1, pos_y1, pos_x2, pos_y2,
      window->GetXMinWindow(), window->GetXMaxWindow(),
      window->GetYMinWindow(), window->GetYMaxWindow()
    );
  } else
    DesenhaLinhaFinal(pos_x1,pos_y1,pos_x2,pos_y2);

}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaLinhaFinal(float pos_x1,float pos_y1,float pos_x2,float pos_y2)
{
  canvas->MoveTo(PegaX(pos_x1),PegaY(pos_y1));
  canvas->LineTo(PegaX(pos_x2),PegaY(pos_y2));
}
//---------------------------------------------------------------------------
void SistemaGrafico::Clipping(bool _clipping)
{
  clipping = _clipping;
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaCurva(Curva* curva)
{
  DesenhaCurvaFwdDiff(curva);
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaCurvaIterativo(Curva* curva)
{
  vector<float*>* vetorCoeficientes = curva->CoeficientesDaCurva();
  float passo = 1.0/curva->PassoRecomendado(window->GetXMaxWindow() - window->GetXMinWindow(), TAM_CANVAS);
  for(unsigned int i=0 ;i<vetorCoeficientes->size();i++){
    float* coeficientes = (*vetorCoeficientes)[i];
    float ax = coeficientes[0];
    float bx = coeficientes[1];
    float cx = coeficientes[2];
    float dx = coeficientes[3];
    float ay = coeficientes[4];
    float by = coeficientes[5];
    float cy = coeficientes[6];
    float dy = coeficientes[7];

    float x0 = dx;
    float y0 = dy;
    for(float t = passo; t<1.0-passo;t+=passo)
    {
      float t2 = t*t;
      float t3 = t2*t;
      float x1 = ax*t3 + bx*t2 + cx*t + dx;
      float y1 = ay*t3 + by*t2 + cy*t + dy;
      if (clipping)
        RecorteLinhaCohenSutherland (
          x0, y0, x1, y1,
          window->GetXMinWindow(), window->GetXMaxWindow(),
          window->GetYMinWindow(), window->GetYMaxWindow()
        );
      else
        DesenhaLinhaFinal(x0,y0,x1,y1);
      x0 = x1;
      y0 = y1;
      delete coeficientes;
    }
  }
  delete vetorCoeficientes;
}
//---------------------------------------------------------------------------
void SistemaGrafico::InsereCurva(Curva* curva)
{
  listaObjetos.push_back(curva);
  RotacionaWindowEixoZ(0.0);
  DesenhaCurva(curva);
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaCurvaFwdDiff(Curva* curva)
{
  int n =curva->PassoRecomendado(window->GetXMaxWindow() - window->GetXMinWindow(), TAM_CANVAS);
  DesenhaCurvaFwdDiff(n, curva->CoeficientesDaCurva());
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaCurvaFwdDiff(int n, vector<float*>* vetorCoeficientes)
{
  Matriz* matrizFwdDiff = Matriz::CriarMatrizFwdDiff(n);
  for(unsigned int i=0 ;i<vetorCoeficientes->size();i++){
    float* coeficientes = (*vetorCoeficientes)[i];
    Matriz matrizCoeficientes(4,1);
    matrizCoeficientes.ColocarElemento(0,0,coeficientes[0]);
    matrizCoeficientes.ColocarElemento(1,0,coeficientes[1]);
    matrizCoeficientes.ColocarElemento(2,0,coeficientes[2]);
    matrizCoeficientes.ColocarElemento(3,0,coeficientes[3]);
    Matriz* MatrizX = Matriz::MultiplicaMatriz(matrizFwdDiff, &matrizCoeficientes);
    matrizCoeficientes.ColocarElemento(0,0,coeficientes[4]);
    matrizCoeficientes.ColocarElemento(1,0,coeficientes[5]);
    matrizCoeficientes.ColocarElemento(2,0,coeficientes[6]);
    matrizCoeficientes.ColocarElemento(3,0,coeficientes[7]);
    Matriz* MatrizY = Matriz::MultiplicaMatriz(matrizFwdDiff, &matrizCoeficientes);
    matrizCoeficientes.ColocarElemento(0,0,coeficientes[8]);
    matrizCoeficientes.ColocarElemento(1,0,coeficientes[9]);
    matrizCoeficientes.ColocarElemento(2,0,coeficientes[10]);
    matrizCoeficientes.ColocarElemento(3,0,coeficientes[11]);
    Matriz* MatrizZ = Matriz::MultiplicaMatriz(matrizFwdDiff, &matrizCoeficientes);
    float x   = MatrizX->GetElemento(0,0);
    float dx  = MatrizX->GetElemento(1,0);
    float d2x = MatrizX->GetElemento(2,0);
    float d3x = MatrizX->GetElemento(3,0);
    float y   = MatrizY->GetElemento(0,0);
    float dy  = MatrizY->GetElemento(1,0);
    float d2y = MatrizY->GetElemento(2,0);
    float d3y = MatrizY->GetElemento(3,0);
    float z   = MatrizZ->GetElemento(0,0);
    float dz  = MatrizZ->GetElemento(1,0);
    float d2z = MatrizZ->GetElemento(2,0);
    float d3z = MatrizZ->GetElemento(3,0);

    delete MatrizX;
    delete MatrizY;
    delete MatrizZ;
    delete coeficientes;
    DesenhaCurvaFwdDiff( x, dx, d2x, d3x,y, dy, d2y, d3y, z, dz, d2z, d3z, n);

  }
  delete matrizFwdDiff;
  delete vetorCoeficientes;
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaCurvaFwdDiff(
        float _x, float _dx, float _d2x, float _d3x,
        float _y, float _dy, float _d2y, float _d3y,
        float _z, float _dz, float _d2z, float _d3z, int n
){

    float x   = _x;
    float dx  = _dx;
    float d2x = _d2x;
    float d3x = _d3x;
    float y   = _y;
    float dy  = _dy;
    float d2y = _d2y;
    float d3y = _d3y;
    float z   = _z;
    float dz  = _dz;
    float d2z = _d2z;
    float d3z = _d3z;

    int i = 0;
    while (i<n){
      float x0 = x;
      float y0 = y;
      float z0 = z;
      i++;
      x   = x   + dx;
      dx  = dx  + d2x;
      d2x = d2x + d3x;
      y   = y   + dy;
      dy  = dy  + d2y;
      d2y = d2y + d3y;
      z   = z   + dz;
      dz  = dz  + d2z;
      d2z = d2z + d3z;
      DesenhaLinha(x0,y0,z0,x,y,z);
    }
}

//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaSuperficie(Superficie* superficie)
{
  int passo = 100;
  int passoS = 20;
  int passoT = 20;
  float delta = 1.0/passo;
  float deltaS = 1.0/(passoS-1);
  float deltaT = 1.0/(passoT-1);

  float s = 0.0;
  for(int i=0;i<passoS;i++) {
    Ponto* p0 = superficie->GeraPonto(s,0.0);
    float t = delta;
    for(int j=1; j<passo; j++) {
      Ponto* p1 = superficie->GeraPonto(s,t);
      DesenhaLinha(p0->GetX(),p0->GetY(),p0->GetZ(),p1->GetX(),p1->GetY(),p1->GetZ());
      delete p0;
      p0 = p1;
      t += delta;
    }
    s +=deltaS;
  }

  float t = 0.0;
  for(int i=0;i<passoT;i++) {
    Ponto* p0 = superficie->GeraPonto(0.0,t);
    float s = delta;
    for(int j=1; j<passo; j++) {
      Ponto* p1 = superficie->GeraPonto(s,t);
      DesenhaLinha(p0->GetX(),p0->GetY(),p0->GetZ(),p1->GetX(),p1->GetY(),p1->GetZ());
      delete p0;
      p0 = p1;
      s += delta;
    }
    t +=deltaT;
  }

}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaSuperficieFwdDif(Superficie* superficie)
{
    int ns = 25;
    int n = 25;
    vector<Matriz*>* matrizesX = superficie->GetMatrizMGMT(EIXO_X);
    vector<Matriz*>* matrizesY = superficie->GetMatrizMGMT(EIXO_Y);
    vector<Matriz*>* matrizesZ = superficie->GetMatrizMGMT(EIXO_Z);

    for(unsigned int i=0; i<matrizesX->size();i++) {
      Matriz* mgmtX = (*matrizesX)[i];
      Matriz* mgmtY = (*matrizesY)[i];
      Matriz* mgmtZ = (*matrizesZ)[i];
      Matriz* matrizFwdDiff = Matriz::CriarMatrizFwdDiff(n);
      Matriz* matrizFwdDiffTransposta = matrizFwdDiff->GeraTransposta();

      Matriz* temp = Matriz::MultiplicaMatriz(matrizFwdDiff, mgmtX);
      Matriz* MatrizDX = Matriz::MultiplicaMatriz(temp, matrizFwdDiffTransposta);
      delete temp;
      temp = Matriz::MultiplicaMatriz(matrizFwdDiff, mgmtY);
      Matriz* MatrizDY = Matriz::MultiplicaMatriz(temp, matrizFwdDiffTransposta);
      delete temp;
      temp = Matriz::MultiplicaMatriz(matrizFwdDiff, mgmtZ);
      Matriz* MatrizDZ = Matriz::MultiplicaMatriz(temp, matrizFwdDiffTransposta);
      delete temp;
      Matriz* MatrizDXTransposta = MatrizDX->GeraTransposta();
      Matriz* MatrizDYTransposta = MatrizDY->GeraTransposta();
      Matriz* MatrizDZTransposta = MatrizDZ->GeraTransposta();

      for (int k=0 ;k<2;k++) { // Horizontal e verical
        for(int i=0;i<ns+1;i++){
          float x = MatrizDX->GetElemento(0,0);
          float dx = MatrizDX->GetElemento(0,1);
          float d2x = MatrizDX->GetElemento(0,2);
          float d3x = MatrizDX->GetElemento(0,3);
          float y = MatrizDY->GetElemento(0,0);
          float dy = MatrizDY->GetElemento(0,1);
          float d2y = MatrizDY->GetElemento(0,2);
          float d3y = MatrizDY->GetElemento(0,3);
          float z = MatrizDZ->GetElemento(0,0);
          float dz = MatrizDZ->GetElemento(0,1);
          float d2z = MatrizDZ->GetElemento(0,2);
          float d3z = MatrizDZ->GetElemento(0,3);

          DesenhaCurvaFwdDiff(x,dx,d2x, d3x,y, dy, d2y, d3y, z, dz, d2z, d3z, n);

          MatrizDX->SomaComLinhaAbaixo();
          MatrizDY->SomaComLinhaAbaixo();
          MatrizDZ->SomaComLinhaAbaixo();
        }
        // Na segunda iteracao trabalhar com a transposta (horizontal e vertical)
        delete MatrizDX;
        delete MatrizDY;
        delete MatrizDZ;
        MatrizDX = MatrizDXTransposta;
        MatrizDY = MatrizDYTransposta;
        MatrizDZ = MatrizDZTransposta;
      }
      delete matrizFwdDiff;
      delete matrizFwdDiffTransposta;
    }
}
//---------------------------------------------------------------------------
void SistemaGrafico::ScanConvertion()
{
  ZerarZBuffer();
  for(unsigned int i=0;i<listaObjetos.size();i++){
    ObjetoTriangularizado* atual = dynamic_cast<ObjetoTriangularizado*>(listaObjetos[i]);
    if(atual){
      atual->FazTransformadaDeViewPort(this);
      window->GetCOP().FazTransformadaDeViewPort(this);
      for(unsigned int x=0; x<pontosDeLuz.size(); x++)
        pontosDeLuz[x]->FazTransformadaDeViewPort(this);
      vector<Triangulo*>& triangulos = atual->getTriangulos();

      for(unsigned int j=0; j<triangulos.size();j++){
        Triangulo* trianguloAtual = triangulos[j];
        Ponto* p0 = trianguloAtual->PontoComMenorY();
        Ponto* p1 = trianguloAtual->PontoComYIntermediario();
        Ponto* p2 = trianguloAtual->PontoComMaiorY();

        int inicio = p0->GetYViewPort();
        int fim = p1->GetYViewPort() >= TAM_TOTAL_VIEWPORT ? TAM_TOTAL_VIEWPORT - 1 : p1->GetYViewPort();

        if (inicio == fim && p2->GetXViewPort() >= p1->GetXViewPort()){
          Ponto* temp = p0;
          p0 = p1;
          p1 = temp;
        }

        bool erro = false;
        float Mx, Mz, mx, mz;
        Mx = (p2->GetYViewPort()-p0->GetYViewPort() == 0) ? 0 : (float)(p2->GetXViewPort()-p0->GetXViewPort())/(p2->GetYViewPort()-p0->GetYViewPort());
        Mz = (p2->GetYViewPort()-p0->GetYViewPort() == 0) ? 0 : (float)(p2->GetZWindow()-p0->GetZWindow())/(p2->GetYViewPort()-p0->GetYViewPort());
        // Passo 1
        mx = (p1->GetYViewPort()-p0->GetYViewPort()) == 0 ? 0 : (float)(p1->GetXViewPort()-p0->GetXViewPort())/(p1->GetYViewPort()-p0->GetYViewPort());
        mz = (p1->GetYViewPort()-p0->GetYViewPort()) == 0 ? 0 : (float)(p1->GetZWindow()-p0->GetZWindow())/(p1->GetYViewPort()-p0->GetYViewPort());

        // Quando for p1 p2 p0 ou p1 p0 p2 ou p2 p1 p0 inverte...
        if (( p0->GetXViewPort() >= p2->GetXViewPort() &&
              p2->GetXViewPort() >= p1->GetXViewPort()) ||
           ( p2->GetXViewPort() >= p0->GetXViewPort() &&
             p0->GetXViewPort() >= p1->GetXViewPort()) ||
           ( (p0->GetXViewPort() >= p1->GetXViewPort() &&
             p1->GetXViewPort() >= p2->GetXViewPort()) && Mx > mx) )
          erro = true;
        else
          erro = false;

        if (p2->GetXViewPort() >= p1->GetXViewPort() &&
             p1->GetXViewPort() >= p0->GetXViewPort() && Mx > mx)
             erro = true;

        float x0 = p0->GetXViewPort();
        float x1 = p0->GetXViewPort();
        float z0 = p0->GetZWindow();
        float z1 = p0->GetZWindow();

        if(erro){
          float temp = Mx;
          Mx = mx;
          mx = temp;

          temp = Mz;
          Mz = mz;
          mz = temp;

        }

        // Otimizao ("pintar" s o que est visvel)
        if (inicio < 0){
          x0 = x0 + abs(inicio)*Mx;
          z0 = z0 + abs(inicio)*Mz;
          x1 = x1 + abs(inicio)*mx;
          z1 = z1 + abs(inicio)*mz;
          inicio=0;
        }
        if (fim >= TAM_TOTAL_VIEWPORT)
          fim = TAM_TOTAL_VIEWPORT-1;

        for(int y = inicio; y <= fim; ++y){
          float t = 0;
          int xInicial = x0 < 0 ? 0 : x0;
          int xFinal = x1 >= TAM_TOTAL_VIEWPORT ? TAM_TOTAL_VIEWPORT - 1 : x1;
          float z1_z0 = z1 - z0;
          float mzHorz = (xFinal-xInicial) == 0 ? 0 : 1 / (float)(xFinal-xInicial);
          for (int x = xInicial; x<= xFinal; ++x){
            float z = z0 + t*z1_z0;
            // Atualiza zBuffer
            if (zBuffer[x][y] == -1.0 || (z < zBuffer[x][y] && z >1.0)){
              zBuffer[x][y] = z;
              trianguloBuffer[x][y] = trianguloAtual;
            }
            t = t + mzHorz;
          }

          x0 = x0 + Mx;
          z0 = z0 + Mz;
          x1 = x1 + mx;
          z1 = z1 + mz;

        }

        // Segundo passo
        Mx = (p2->GetYViewPort()-p0->GetYViewPort() == 0) ? 0 : -(float)(p2->GetXViewPort()-p0->GetXViewPort())/(p2->GetYViewPort()-p0->GetYViewPort());
        Mz = (p2->GetYViewPort()-p0->GetYViewPort() == 0) ? 0 : -(float)(p2->GetZWindow()-p0->GetZWindow())/(p2->GetYViewPort()-p0->GetYViewPort());
        mx = (p2->GetYViewPort()-p1->GetYViewPort()) == 0 ? 0 : -(float)(p2->GetXViewPort()-p1->GetXViewPort())/(p2->GetYViewPort()-p1->GetYViewPort());
        mz = (p2->GetYViewPort()-p1->GetYViewPort()) == 0 ? 0 : -(float)(p2->GetZWindow()-p1->GetZWindow())/(p2->GetYViewPort()-p1->GetYViewPort());

        x0 = p2->GetXViewPort();
        x1 = p2->GetXViewPort();
        z0 = p2->GetZWindow();
        z1 = p2->GetZWindow();

        inicio = p2->GetYViewPort();

        if(erro){
          float temp = Mx;
          Mx = mx;
          mx = temp;

          temp = Mz;
          Mz = mz;
          mz = temp;

        }

        // Otimizao ("pintar" s o que est visvel)
        if (fim < 0)
          fim=0;
        if (inicio >= TAM_TOTAL_VIEWPORT){
          int delta = TAM_TOTAL_VIEWPORT - inicio -1;
          x0 = x0 + abs(delta)*Mx;
          z0 = z0 + abs(delta)*Mz;
          x1 = x1 + abs(delta)*mx;
          z1 = z1 + abs(delta)*mz;
          inicio = TAM_TOTAL_VIEWPORT-1;
        }

        for(int y = inicio; y >= fim; --y){
          float t = 0;
          int xInicial = x0 < 0 ? 0 : x0;
          int xFinal = x1 >= TAM_TOTAL_VIEWPORT ? TAM_TOTAL_VIEWPORT - 1 : x1;
          float Z1_Z0 = z1 - z0;
          float mzHorz = (xFinal-xInicial) == 0 ? 0 : 1 / (float)(xFinal-xInicial);
          for (int x = xInicial; x<=xFinal; ++x){
            float z = z0 + t*Z1_Z0;
            // Atualiza zBuffer
            if (zBuffer[x][y] == -1.0 || (z < zBuffer[x][y] && z >1.0)){
              zBuffer[x][y] = z;
              trianguloBuffer[x][y] = trianguloAtual;
            }
            t = t + mzHorz;
          }
          x0 = x0 + Mx;
          z0 = z0 + Mz;
          x1 = x1 + mx;
          z1 = z1 + mz;
        }
      }
    }
  }
  FazIluminacao();
}
//---------------------------------------------------------------------------
void SistemaGrafico::ZerarZBuffer(){
  for(int x=0;x<TAM_TOTAL_VIEWPORT;x++)
    for(int y=0;y<TAM_TOTAL_VIEWPORT;y++){
      zBuffer[x][y]=-1.0;
      colorBuffer[x][y][0] = 255;
      colorBuffer[x][y][1] = 255;
      colorBuffer[x][y][2] = 255;
    }

}
//---------------------------------------------------------------------------
void SistemaGrafico::FazIluminacao()
{
  Ponto* pCOP = &window->GetCOP();
  PontoSimples cop(pCOP->GetXViewPort(),pCOP->GetYViewPort(), pCOP->GetZWindow());

  for(int x=0; x<TAM_TOTAL_VIEWPORT; x++)
    for(int y=0; y<TAM_TOTAL_VIEWPORT; y++)
      if(zBuffer[x][y] != -1)
        AplicaIluminacao(trianguloBuffer[x][y], PontoSimples(x,y,zBuffer[x][y]), cop);
}
//---------------------------------------------------------------------------
void SistemaGrafico::AplicaIluminacao(
  Triangulo* triangulo,
  PontoSimples& ponto,
  PontoSimples& cop
)
{
  ObjetoTriangularizado* obj = triangulo->GetPai();

  Cor* corAmbRef = obj->getAmbRefCoef();
  Cor* corDiffRef = obj->getDiffRefCoef();
  Cor* corSpecRef = obj->getSpecRefCoef();
  float shineRef = obj->getShineCoef();

  // Componente Ambinete
  float IaaR = 0;
  float IaaG = 0;
  float IaaB = 0;

  if (componenteAmbiente) {
    IaaR = corAmbRef->GetR() * corLuzAmbiente.GetR();
    IaaG = corAmbRef->GetG() * corLuzAmbiente.GetG();
    IaaB = corAmbRef->GetB() * corLuzAmbiente.GetB();
  }

  float IR = 0.0;
  float IG = 0.0;
  float IB = 0.0;

  for(unsigned int i=0;i<pontosDeLuz.size();i++) {
    Ponto* p = pontosDeLuz[i];
    PontoSimples pontoDeLuz(p->GetXViewPort(),p->GetYViewPort(), p->GetZWindow());
    Cor corPontoLuz = p->GetCor();

    // Componente especular
    float IsR = 0;
    float IsG = 0;
    float IsB = 0;
    PontoSimples vetorDirecaoLuz = (ponto - pontoDeLuz);
    PontoSimples vetorRaioLuz = (pontoDeLuz - ponto);
    PontoSimples normal = *triangulo->GetVetorSimples();
/*
    Ponto* ponto1 = triangulo->GetP1();
    Ponto* ponto2 = triangulo->GetP2();
    Ponto* ponto3 = triangulo->GetP3();
    PontoSimples p0(ponto1->GetXWindow(), ponto1->GetYWindow(), ponto1->GetZWindow());
    PontoSimples p1(ponto2->GetXWindow(), ponto2->GetYWindow(), ponto2->GetZWindow());
    PontoSimples p2(ponto3->GetXWindow(), ponto3->GetYWindow(), ponto3->GetZWindow());
    PontoSimples& perpendicular = CrossProduct(p1-p0, p2-p0);
    if(perpendicular.Z() > 0)
      perpendicular = CrossProduct(p0-p1, p2-p1);
    if(perpendicular.Z() > 0)
      perpendicular = CrossProduct(p0-p2, p1-p2);

    bool voltadoParaLuz = false;

    float anguloPerpDirecaoLuz = AnguloEntre(perpendicular, vetorDirecaoLuz);
    if(anguloPerpDirecaoLuz < 90)
      voltadoParaLuz = true;
*/
    // Verifica se algum outro triangulo est fazendo sombra
    bool voltadoParaLuz = true;
    bool sombra = false;
    if(usarSombra){
      for(unsigned int j=0;j<listaObjetos.size();j++){
        ObjetoTriangularizado* atual = dynamic_cast<ObjetoTriangularizado*>(listaObjetos[j]);
        if(atual){
          vector<Triangulo*>& triangulos = atual->getTriangulos();
          for(unsigned int k=0; k<triangulos.size();k++){
            Triangulo* trianguloAtual = triangulos[k];
            if(triangulo!=trianguloAtual && !sombra)
              if(IntersectRayTriangle( vetorRaioLuz, *trianguloAtual,ponto)==1)
                sombra = true;
          }
        }
      }
    }

    if (componenteEspecular && voltadoParaLuz && !sombra) {
      PontoSimples vetorDirecaoObs = (cop - ponto);
      float angulo1 = AnguloEntre(vetorDirecaoLuz, normal);
      float angulo2 = AnguloEntre(vetorDirecaoObs, normal);
      float anguloFinal = angulo1 - angulo2;
      float temp = anguloFinal == 0 ? 0 : pow(cos(anguloFinal*pi_180),shineRef);

      IsR = corSpecRef->GetR() * temp;
      IsG = corSpecRef->GetG() * temp;
      IsB = corSpecRef->GetB() * temp;
    }

    // Componente Difuso
    float IdR = 0.0;
    float IdG = 0.0;
    float IdB = 0.0;
    if (componenteDifusa && voltadoParaLuz && !sombra){
      float NL1 = CosAnguloEntre(normal, vetorDirecaoLuz);
      float NL2 = CosAnguloEntre(normal, vetorRaioLuz);
      float NL = NL1 > NL2 ? NL1 : NL2;
      if(NL > 0 && !sombra) {
        IdR = corDiffRef->GetR() * corPontoLuz.GetR() * NL;
        IdG = corDiffRef->GetG() * corPontoLuz.GetG() * NL;
        IdB = corDiffRef->GetB() * corPontoLuz.GetB() * NL;
      }
    }

 // Final...
 //    float distancia = pontoDeLuz.Distancia(ponto);
 //   float fator = distancia*fatorK;
    float fator = 1;

    IR = IR + (fator == 0 ? 0 : IaaR + (IdR + IsR) / (fator));
    IG = IG + (fator == 0 ? 0 : IaaG + (IdG + IsG) / (fator));
    IB = IB + (fator == 0 ? 0 : IaaB + (IdB + IsB) / (fator));
  }


  if(IR>1.0)
    IR = 1.0;
  if(IG>1.0)
    IG = 1.0;
  if(IB>1.0)
    IB = 1.0;

  int posY = TAM_TOTAL_VIEWPORT-(int)ponto.Y()-1;
  int posX = (int)ponto.X();

  colorBuffer[posY][posX][0] = IB*255;
  colorBuffer[posY][posX][1] = IG*255;
  colorBuffer[posY][posX][2] = IR*255;

}
//---------------------------------------------------------------------------
// intersect_RayTriangle(): intersect a ray with a 3D triangle
//    Input:  a ray R, and a triangle T
//    Output: *I = intersection point (when it exists)
//    Return: -1 = triangle is degenerate (a segment or point)
//             0 = disjoint (no intersect)
//             1 = intersect in unique point I1
//             2 = are in the same plane
int SistemaGrafico::IntersectRayTriangle(
  PontoSimples& vetorDirecaoLuz,
  Triangulo& triangulo,
  PontoSimples& pontoTriangulo
)
{
    float     r, a, b;             // params to calc ray-plane intersect

    Ponto* ponto1 = triangulo.GetP1();
    Ponto* ponto2 = triangulo.GetP2();
    Ponto* ponto3 = triangulo.GetP3();
    PontoSimples p0(ponto1->GetXViewPort(), ponto1->GetYViewPort(), ponto1->GetZWindow());
    PontoSimples p1(ponto2->GetXViewPort(), ponto2->GetYViewPort(), ponto2->GetZWindow());
    PontoSimples p2(ponto3->GetXViewPort(), ponto3->GetYViewPort(), ponto3->GetZWindow());
    PontoSimples& v0 = p0;
    PontoSimples& u = p1-p0;
    PontoSimples& v = p2-p0;
    PontoSimples& vetorPerpendicular = CrossProduct(u, v);
/*
    if(vetorPerpendicular.Z() > 0){
      PontoSimples& u = p0-p1;
      PontoSimples& v = p2-p1;
      vetorPerpendicular = CrossProduct(u, v);
      v0 = p1;
    }if(vetorPerpendicular.Z() > 0){
      PontoSimples& u = p0-p2;
      PontoSimples& v = p1-p2;
      vetorPerpendicular = CrossProduct(u, v);
      v0 = p2;
    }
*/
    PontoSimples w0(pontoTriangulo - v0); // Ponto no triangulo (viewport!!??)
    a = -DotProduct(vetorPerpendicular,w0);
    b = DotProduct(vetorPerpendicular,vetorDirecaoLuz);
    if (fabs(b) < 0.0001) {     // ray is parallel to triangle plane
        if (a == 0)                // ray lies in triangle plane
            return 2;
        else return 0;             // ray disjoint from plane
    }

    // get intersect point of ray with triangle plane
    r = a / b;
    if (r <= 0.0)                   // ray goes away from triangle
        return 0;                  // => no intersect
    // for a segment, also test if (r > 1.0) => no intersect

    // intersect point of ray and plane
    PontoSimples& temp = vetorDirecaoLuz*r;
    PontoSimples& i = pontoTriangulo + temp;

    // is I inside T?
    float    uu, uv, vv, wu, wv, D;
    uu = DotProduct(u,u);
    uv = DotProduct(u,v);
    vv = DotProduct(v,v);
    PontoSimples& w = i - v0;
    wu = DotProduct(w,u);
    wv = DotProduct(w,v);
    D = uv * uv - uu * vv;

    // get and test parametric coords
    float s, t;
    s = (uv * wv - vv * wu) / D;
    if (s < 0.0 || s > 1.0)        // I is outside T
        return 0;
    t = (uv * wu - uu * wv) / D;
    if (t < 0.0 || (s + t) > 1.0)  // I is outside T
        return 0;

    return 1;                      // I is in T
}
//---------------------------------------------------------------------------
PontoSimples SistemaGrafico::CrossProduct(PontoSimples& p1, PontoSimples& p2)
{
  float x1 = p1.X();
  float x2 = p2.X();
  float y1 = p1.Y();
  float y2 = p2.Y();
  float z1 = p1.Z();
  float z2 = p2.Z();

  float x = y1*z2 - z1*y2;
  float y = z1*x2 - x1*z2;
  float z = x1*y2 - y1*x2;

  return PontoSimples(x,y,z);

}
//---------------------------------------------------------------------------
float SistemaGrafico::DotProduct(PontoSimples& p1, PontoSimples& p2)
{
  return p1.X()*p2.X() + p1.Y()*p2.Y() + p1.Z()*p2.Z();
}

//---------------------------------------------------------------------------
float SistemaGrafico::Normal(PontoSimples& vetor)
{
  return sqrt(vetor.X()*vetor.X() + vetor.Y()*vetor.Y() + vetor.Z()*vetor.Z());
}
//---------------------------------------------------------------------------
float SistemaGrafico::CosAnguloEntre(PontoSimples& p1,PontoSimples& p2)
{
   float dot = DotProduct(p1,p2);
   float normA = p1.Normal();
   float normB = p2.Normal();
   float angle = dot/(normA*normB);

   if ( angle > 1 || angle < -1)
     angle = 1;

   return angle;
}
//---------------------------------------------------------------------------
float SistemaGrafico::AnguloEntre(PontoSimples& p1,PontoSimples& p2)
{
  float angRad = acos(CosAnguloEntre(p1, p2));
  return (angRad)*_180_pi;
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaRayTracing()
{
  // API do Windows, muito mais rpido que o Canvas do Borland
  HDC hDC = canvasReal->Handle;
  BITMAPINFO info;
  info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  info.bmiHeader.biWidth = TAM_TOTAL_VIEWPORT-1;
  info.bmiHeader.biHeight = TAM_TOTAL_VIEWPORT-1;
  info.bmiHeader.biPlanes = 1;
  info.bmiHeader.biBitCount = 24;
  info.bmiHeader.biCompression = BI_RGB;


  StretchDIBits(hDC,
    0,0,TAM_TOTAL_VIEWPORT-1,TAM_TOTAL_VIEWPORT-1,  // screen coordinates
    0,0,TAM_TOTAL_VIEWPORT-1,TAM_TOTAL_VIEWPORT-1,  // bitmap coordinates
    &colorBuffer,&info,DIB_RGB_COLORS,SRCCOPY);

}
//---------------------------------------------------------------------------
void SistemaGrafico::SetComponenteDifuso(bool valor)
{
  componenteDifusa = valor;
}
//---------------------------------------------------------------------------
void SistemaGrafico::SetComponenteAmbiente(bool valor)
{
  componenteAmbiente = valor;
}
//---------------------------------------------------------------------------
void SistemaGrafico::SetComponenteEspecular(bool valor)
{
  componenteEspecular = valor;
}
//---------------------------------------------------------------------------
void SistemaGrafico::InsereSuperficie(Superficie* superfice)
{
  listaObjetos.push_back(superfice);
  RotacionaWindowEixoZ(0.0);
  DesenhaSuperficieFwdDif(superfice);

}
//---------------------------------------------------------------------------

void SistemaGrafico::Remove(int pos)
{
  vector<ObjetoGrafico*>::iterator it = listaObjetos.begin();
  for(int i=0;i<pos;i++){it++;}
  ObjetoGrafico* objeto = listaObjetos[pos];
  Ponto* ponto = dynamic_cast<Ponto*>(objeto);
  if (ponto) // Pode ser um ponto de luz
    if(ponto->GetPontoLuz()){
      vector<Ponto*>::iterator itPonto = pontosDeLuz.begin();
      for(unsigned int i=0; i<pontosDeLuz.size(); i++){
        if(pontosDeLuz[i] == ponto)
          pontosDeLuz.erase(itPonto);
        itPonto++;
      }
    }

  delete listaObjetos[pos];
  listaObjetos.erase(it);
  DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::AbreDoArquivo(const char* nomeArquivo)
{
    ifstream arquivo (nomeArquivo, ios::binary);
    int tamanho;
    int tipo;
    int versao;
    arquivo >> versao;
    arquivo >> tamanho;
    ObjetoGrafico* objeto;
    for(int i=0;i<tamanho;i++) {
      arquivo >> tipo;
      switch (tipo) {
        case 0: {
          Ponto* ponto = Ponto::LeDoStream(arquivo, versao);
          if (ponto->GetPontoLuz())
            pontosDeLuz.push_back(ponto);
          objeto = ponto;
          }
        break;
        case 1:
          objeto = Reta::LeDoStream(arquivo, versao);
        break;
        case 2:
          objeto = WireFrame::LeDoStream(arquivo, versao);
        break;
        case 3:
          objeto = Curva::LeDoStream(arquivo, versao);
        break;
        case 4:
          objeto = Superficie::LeDoStream(arquivo, versao);
        break;
        case 5:
          objeto = ObjetoTriangularizado::LeDoStream(arquivo, versao);
        break;

      }
      listaObjetos.push_back(objeto);
      objeto->SetCor(clBlack);
    }
    arquivo.close();
    Projecao(0,0,0);
    DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::SalvaNoArquivo(const char* nomeArquivo)
{
  ofstream arquivo (nomeArquivo, ios::binary);
  arquivo << VERSAO_ATUAL << " ";
  arquivo << listaObjetos.size() << " ";

  for(unsigned int i=0;i<listaObjetos.size();i++) {
    ObjetoGrafico* atual = listaObjetos[i];
    Ponto* ponto = dynamic_cast<Ponto*> (atual);
    if (ponto) {
      arquivo << 0 << " ";
      ponto->SalvaNoStream(arquivo);
    }
    Reta* reta = dynamic_cast<Reta*> (atual);
    if (reta) {
      arquivo << 1 << " ";
      reta->SalvaNoStream(arquivo);
    }
    WireFrame* wireframe = dynamic_cast<WireFrame*>(atual);
    if(wireframe){
      arquivo << 2 << " ";
      wireframe->SalvaNoStream(arquivo);
    }
    Curva* curva = dynamic_cast<Curva*>(atual);
    if (curva) {
      arquivo << 3 << " ";
      curva->SalvaNoStream(arquivo);
    }
    Superficie* superficie = dynamic_cast<Superficie*>(atual);
    if (superficie) {
      arquivo << 4 << " ";
      superficie->SalvaNoStream(arquivo);
    }
    ObjetoTriangularizado* triang = dynamic_cast<ObjetoTriangularizado*>(atual);
    if (triang) {
      arquivo << 5 << " ";
      triang->SalvaNoStream(arquivo);
    }

  }
  arquivo.close();
}
//---------------------------------------------------------------------------
vector<ObjetoGrafico*>& SistemaGrafico::GetListaObjetos()
{
  return listaObjetos;
}
//---------------------------------------------------------------------------
void SistemaGrafico::setProjecaoParalela(bool _projecaoParalela)
{
  projecaoParalela = _projecaoParalela;
  Projecao(0,0,0);
  DesenhaTudo();
}
//---------------------------------------------------------------------------
void SistemaGrafico::AdicionaPontoDeLuz(Ponto* pontoLuz)
{
  pontosDeLuz.push_back(pontoLuz);
  listaObjetos.push_back(pontoLuz);  
}
//---------------------------------------------------------------------------
void SistemaGrafico::SetCorLuzAmbiente(Cor corAmbiente)
{
  corLuzAmbiente = corAmbiente;
}
//---------------------------------------------------------------------------
void SistemaGrafico::AdicionaObjetoTriangularizado(ObjetoTriangularizado* obj)
{
  listaObjetos.push_back(obj);
  DesenhaObjetoTriangularizado(obj);
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaTriangulo(Triangulo* triangulo)
{
  DesenhaWireFrame(triangulo);
}
//---------------------------------------------------------------------------
void SistemaGrafico::DesenhaObjetoTriangularizado(ObjetoTriangularizado* obj)
{
  vector<Triangulo*>& vetorTriangulos = obj->getTriangulos();
  for(unsigned int i=0;i<vetorTriangulos.size();i++)
    DesenhaTriangulo(vetorTriangulos[i]);
}
//---------------------------------------------------------------------------

void SistemaGrafico::setPontosWindow(Ponto* p1, Ponto* p2, Ponto* p3, Ponto* p4)
{
  delete window;
  window = new Window(p1, p2, p3, p4);
}
//---------------------------------------------------------------------------
void SistemaGrafico::setVPNWindow(Ponto* VPN)
{
  window->SetVPN(VPN);
}
//---------------------------------------------------------------------------
void SistemaGrafico::SetFatorK(float _fator)
{
  fatorK = _fator;
}
//---------------------------------------------------------------------------
//  CLIPPING
//---------------------------------------------------------------------------
bool SistemaGrafico::Vazio(outcode codigo[])
/* retorna true se todos os outcodes em codigo sao false,
   e false caso contrario */
{
  bool retorno = true;
  for(int i=LEFT;i<=TOP;i++){
    if (codigo[i]==true)
      retorno = false;
  }
  return retorno;
}
//---------------------------------------------------------------------------
bool SistemaGrafico::InterseccaoVazia(outcode codigo0[], outcode codigo1[])
/* retorna true se a interseccao logica entre codigo1 e codigo 2 e vazia, e false c.c. */
{
  bool retorno = true;
  for(int i=LEFT; i<=TOP;i++){
    if ((codigo0[i] & codigo1[i]))
      retorno = false;
  }
  return retorno;
}
//---------------------------------------------------------------------------
bool SistemaGrafico::igual(outcode codigo0[], outcode codigo1[])
/* retorna true se codigo0 e codigo 1 sao iguais, e false c.c. */
{
  bool retorno = true;
  for(int i=LEFT; i<=TOP;i++){
    if (codigo0[i]!=codigo1[i])
      retorno = false;
  }
  return retorno;
}
//---------------------------------------------------------------------------
void SistemaGrafico::CalculaOutCode(float x,float y,outcode codigo[], float xmin, float xmax, float ymin, float ymax)
/* calcula outcode para ponto (x,y) */
{
  for (int i = LEFT; i <= TOP; i++ )
    codigo[i] = false;
  if(x<xmin)
    codigo[LEFT]=true;
  if(x>xmax)
    codigo[RIGHT]=true;
  if(y>ymax)
    codigo[TOP]=true;
  if(y<ymin)
    codigo[BOTTOM]=true;
}
//---------------------------------------------------------------------------
void SistemaGrafico::RecorteLinhaCohenSutherland(
  float x0, float y0,
  float x1, float y1,
  float xmin, float xmax,
  float ymin, float ymax
)
/* algoritmo de recorte de Cohen-Sutherland para linha P0(x0,y0) a P1(x1,y1), e retangulo de recorte com diagonal de (xmin,ymin) a (xmax,ymax). */
{
  bool aceito, pronto;
  outcode outcode0[4], outcode1 [4], *outcodeOut;
  /* outcodes para P0, P1 e quaisquer outros pontos que estao fora do retangulo de recorte */
  float x, y, _x0,_y0,_x1,_y1;

  _x0 = x0;
  _y0 = y0;
  _x1 = x1;
  _y1 = y1;

  aceito = false;
  pronto = false;
  CalculaOutCode(x0,y0,outcode0,xmin, xmax, ymin, ymax);
  CalculaOutCode(x1 ,y1 ,outcode1,xmin, xmax, ymin, ymax);
  do {
    if (Vazio(outcode0) && Vazio(outcode1)) {/* aceitacao trivial e sai */
      aceito = true;
      pronto = true;
    } else if (!InterseccaoVazia(outcode0,outcode1)) { /* rejeicao trivial e sai */
      pronto = true;
    } else {
      /* ambos os testes falharam, entao calcula o segmento de reta a recortar:
        a partir de um ponto externo a uma interseccao com a aresta de recorte
        pelo menos um ponto esta fora do retangulo de recorte; seleciona-o */
      outcodeOut= Vazio(outcode0) ? outcode1 : outcode0;
      /* acha agora o ponto de interseccao, usando as formulas:
        y = y0 + inclinacao * (x-x0), x = x0 + ( 1 /inclinacao) * (y-y0) */

      if (outcodeOut[TOP]){ /* divide a linha no topo do retangulo de recorte */
        x = _x0 + (_x1-_x0) * (ymax-_y0)/(_y1-_y0);
        y = ymax;
      } else if (outcodeOut[BOTTOM]){/* divide a linha na base do retangulo de recorte */
        x = _x0 + (_x1-_x0) * (ymin-_y0)/(_y1-_y0);
        y = ymin;
      } else if (outcodeOut[RIGHT]) {/* divide na aresta direita do retangulo de recorte */
        y = _y0 + (_y1 -_y0) * (xmax-_x0)/(_x1-_x0);
        x = xmax;
      } else if (outcodeOut[LEFT]){ /* divide  na aresta esquerda do retangulo de recorte */
        y = _y0 + (_y1 -_y0) * (xmin-_x0)/(_x1-_x0);
        x = xmin;
      }

      if (igual(outcodeOut,outcode0)) {
        _x0 = x;
        _y0 = y;
        CalculaOutCode(_x0,_y0,outcode0,xmin, xmax, ymin, ymax);
      } else {
        _x1 = x;
        _y1 = y;
        CalculaOutCode(_x1,_y1,outcode1, xmin, xmax, ymin, ymax);
      }
    }
  } while (!pronto);

  if (aceito)
      DesenhaLinhaFinal(_x0,_y0,_x1,_y1);
}

//---------------------------------------------------------------------------
#pragma package(smart_init)

