#ifndef USisGraficoH
#define USisGraficoH

#include <vector.h>
#include <vcl.h>
#include <fstream.h>

#include "PontoSimples.h"
#include "Cor.h"

class Reta;
class Ponto;
class WireFrame;
class ObjetoGrafico;
class ObjetoTriangularizado;
class Window;
class Curva;
class Superficie;
class PontoDeLuz;
class Triangulo;

#define TAM_MARGEM_CLIPPING 15
#define TAM_CANVAS 410
#define TAM_TOTAL_VIEWPORT 440

// Versoes do arquivo
#define VERSAO_ATUAL       201

// Controle de versao do arquivo
#define VERSAO_08052004    200
#define VERSAO_19062004    201

//---------------------------------------------------------------------------
class SistemaGrafico
{
  public:
    SistemaGrafico(TCanvas* _canvas);
    ~SistemaGrafico();
    void InsereReta(Reta* reta);
    void InserePonto(Ponto* ponto);
    void InsereWireFrame(WireFrame* wireframe);
    void ZoomIn(int passo);
    void ZoomOut(int passo);
    void Mover(int passo, int d);
    void DesenhaTudo();
    void AplicaTransformacao(
      float* translacao,
      float* escalonamento,
      float* rotacao,
      int posicao
    );
    void RotacionaWindowEixoX(float valor);
    void RotacionaWindowEixoY(float valor);
    void RotacionaWindowEixoZ(float valor);
    void Clipping(bool _clipping);
    void InsereCurva(Curva* curva);
    void InsereSuperficie(Superficie* superficie);
    void AbreDoArquivo(const char* nomeArquivo);
    void SalvaNoArquivo(const char* nomeArquivo);
    void Remove(int posicao);
    void setProjecaoParalela(bool _projecaoParalela);
    vector<ObjetoGrafico*>& GetListaObjetos();
    void AdicionaPontoDeLuz(Ponto* pontoLuz);
    void SetCorLuzAmbiente(Cor corAmbiente);
    void AdicionaObjetoTriangularizado(ObjetoTriangularizado* obj);
    void setPontosWindow(Ponto* p1, Ponto* p2, Ponto* p3, Ponto* p4);
    void setVPNWindow(Ponto* VPN);
    int PegaX(float XW);
    int PegaY(float YW);
    void RayTracing(bool valor);
    Window* GetWindow();
    bool ProjecaoParalela();
    void SetFatorK(float _fator);
    Cor GetCorLuzAmbiente(){ return corLuzAmbiente;};
    void SetComponenteDifuso(bool valor);
    void SetComponenteAmbiente(bool valor);
    void SetComponenteEspecular(bool valor);
    void SetSombra(bool valor){usarSombra = valor;}
    void Animacao();
    void AtivaAnimacao(int pos);

  private:
    typedef enum {LEFT= 0, RIGHT = 1 , BOTTOM = 2, TOP = 3} aresta;
    typedef bool outcode;
    void DesenhaWireFrame(WireFrame* wireframe);

    void DesenhaCurvaFwdDiff(
            float _x, float _dx, float _d2x, float _d3x,
            float _y, float _dy, float _d2y, float _d3y,
            float _z, float _dz, float _d2z, float _d3z, int n
    );

    int IntersectRayTriangle(
      PontoSimples& vetorDirecaoLuz,
      Triangulo& triangulo,
      PontoSimples& pontoLuz
    );

    void DesenhaCurva(Curva* curva);
    void DesenhaCurvaFwdDiff(Curva* curva);
    void DesenhaCurvaFwdDiff(int n, vector<float*>* vetorCoeficientes);
    void DesenhaSuperficie(Superficie* superficie);
    void DesenhaSuperficieFwdDif(Superficie* superficie);
    void DesenhaCurvaIterativo(Curva* curva);
    void Projecao(float valorX,float valorY,float valorZ);
    bool Vazio(outcode codigo[]);
    bool InterseccaoVazia(outcode codigo0[], outcode codigo1[]);
    bool igual(outcode codigo0[], outcode codigo1[]);
    void CalculaOutCode(float x,float y,outcode codigo[], float xmin, float xmax, float ymin, float ymax);
    void RecorteLinhaCohenSutherland(
      float x0, float y0,
      float x1, float y1,
      float xmin,float ymin,
      float xmax,float ymax
    );
   void DesenhaLinha(float x0,float y0, float z0, float x1,float y1, float z1);
   void DesenhaLinhaFinal(float x0,float y0,float x1,float y1);
   bool ClippingDePonto(float x,float y);
   void ScanConvertion();
   void SistemaGrafico::ScanConvertion(
     float _Mx, float _Mz,
     float _mx, float _mz,
     float _x0, float _x1,
     float _z0, float _z1
   );
   void DesenhaObjetoTriangularizado(ObjetoTriangularizado* obj);
   void DesenhaTriangulo(Triangulo* triangulo);
   void DesenhaRayTracing();
   void ZerarZBuffer();
   void AplicaIluminacao(
     Triangulo* triangulo,
     PontoSimples& ponto,
     PontoSimples& cop
   );
   void FazIluminacao();
   PontoSimples CrossProduct(PontoSimples& p1, PontoSimples& p2);
   float DotProduct(PontoSimples& p1, PontoSimples& p2);
   float Normal(PontoSimples& vetor);
   float CosAnguloEntre(PontoSimples& p1,PontoSimples& p2);
   float AnguloEntre(PontoSimples& p1,PontoSimples& p2);

   TCanvas* canvasReal;
   Graphics::TBitmap* bitmapTemp;
   TCanvas* canvas;
   vector<ObjetoGrafico*> listaObjetos;
   Window* window;
   bool clipping;
   bool projecaoParalela;
   Cor corLuzAmbiente;
   vector<Ponto*> pontosDeLuz;
   float zBuffer[TAM_TOTAL_VIEWPORT][TAM_TOTAL_VIEWPORT];
   unsigned char colorBuffer[TAM_TOTAL_VIEWPORT][TAM_TOTAL_VIEWPORT][3];
//   PontoSimples stencilBuffer[TAM_TOTAL_VIEWPORT][TAM_TOTAL_VIEWPORT];
   Triangulo* trianguloBuffer[TAM_TOTAL_VIEWPORT][TAM_TOTAL_VIEWPORT];
   bool rayTracing;
   float fatorK;
   bool componenteAmbiente;
   bool componenteDifusa;
   bool componenteEspecular;
   const float pi_180;
   const float _180_pi;
   bool usarSombra;

};

#endif
