//---------------------------------------------------------------------------
#pragma hdrstop

#include <Math.h>
#include <Values.h>

#include "Window.h"
#include "Ponto.h"
#include "Matriz.h"


//---------------------------------------------------------------------------
Window::Window( float xWMin, float xWMax, float yWMin, float yWMax)
 : WireFrame("Window")
{
  p1 = new Ponto("p1", xWMin, yWMax, -1000.0);
  p2 = new Ponto("p2", xWMin, yWMin, -1000.0);
  p3 = new Ponto("p3", xWMax, yWMin, -1000.0);
  p4 = new Ponto("p4", xWMax, yWMax, -1000.0);
  AdicionaPonto(p1);
  AdicionaPonto(p2);
  AdicionaPonto(p3);
  AdicionaPonto(p4);
  angulo = 0.0;
  anguloX = 90.0;
  anguloY = 90.0;
  anguloZ = 0.0;
  distanciaCOP = 800;
  COP = Centro();
  COP->setNome("COP");
  COP->SetZ(COP->GetZ()-distanciaCOP);
  minimo = MINFLOAT*10000;
  maximo = MAXFLOAT*0.95;
}
//---------------------------------------------------------------------------
Window::Window(Ponto* p1, Ponto* p2, Ponto* p3, Ponto* p4)
: WireFrame("Window")
{
  AdicionaPonto(p1);
  AdicionaPonto(p2);
  AdicionaPonto(p3);
  AdicionaPonto(p4);
}
//---------------------------------------------------------------------------
float Window::GetXMinWindow()
{
  return p1->GetXWindow();
}
//---------------------------------------------------------------------------
float Window::GetXMaxWindow()
{
  return p3->GetXWindow();
}
//---------------------------------------------------------------------------
float Window::GetYMinWindow()
{
  return p2->GetYWindow();
}
//---------------------------------------------------------------------------
float Window::GetYMaxWindow()
{
  return p4->GetYWindow();
}
//---------------------------------------------------------------------------
void Window::SetXMinWindow(float valor)
{
  p1->SetXWindow(valor);
  p2->SetXWindow(valor);
}
//---------------------------------------------------------------------------
void Window::SetXMaxWindow(float valor)
{
  p3->SetXWindow(valor);
  p4->SetXWindow(valor);
}
//---------------------------------------------------------------------------
void Window::SetYMinWindow(float valor)
{
  p2->SetYWindow(valor);
  p3->SetYWindow(valor);
}
//---------------------------------------------------------------------------
void Window::SetYMaxWindow(float valor)
{
  p1->SetYWindow(valor);
  p4->SetYWindow(valor);
}
//---------------------------------------------------------------------------
float Window::GetAngulo()
{
  return angulo;
}
//---------------------------------------------------------------------------
float Window::GetAnguloX()
{
  return anguloX;
}
//---------------------------------------------------------------------------
float Window::GetAnguloY()
{
  return anguloY;
}
//---------------------------------------------------------------------------
float Window::GetAnguloZ()
{
  return anguloZ;
}
//---------------------------------------------------------------------------
void Window::SetAngulo(float _angulo)
{
  angulo = _angulo;
}
//---------------------------------------------------------------------------
void Window::SetAnguloX(float _anguloX)
{
  anguloX = _anguloX;
}
//---------------------------------------------------------------------------
void Window::SetAnguloZ(float _anguloZ)
{
  anguloZ = _anguloZ;
}
//---------------------------------------------------------------------------
void Window::SetAnguloY(float _anguloY)
{
  anguloY = _anguloY;
}
//---------------------------------------------------------------------------
void Window::SetVPN(Ponto* VPN)
{
  this->VPN = VPN;
}
//---------------------------------------------------------------------------
bool Window::ZoomIn(float passo)
{
  if (GetXMaxWindow()-GetXMinWindow()<1.0e-7 ||
    GetYMaxWindow()-GetYMinWindow()<1.0e-7)
    return false;

  float porcent = passo/100;
  float pixelsX = (GetXMaxWindow()-GetXMinWindow())*porcent;
  float pixelsY = (GetYMaxWindow()-GetYMinWindow())*porcent;

  SetXMaxWindow(GetXMaxWindow() - pixelsX);
  SetYMaxWindow(GetYMaxWindow() - pixelsY);
  SetXMinWindow(GetXMinWindow() + pixelsX);
  SetYMinWindow(GetYMinWindow() + pixelsY);

  // Atualiza as coordenadas de mundo
  float escalona[3];
  escalona[0] = 1- 2*porcent;
  escalona[1] = 1- 2*porcent;
  escalona[2] = 1- 2*porcent;
  float translada[3];
  Ponto* centro = Centro();
  translada[0] = -centro->GetX();
  translada[1] = -centro->GetY();
  translada[2] = -centro->GetZ();

  Matriz* matrizTransladaCentro =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
  Matriz* matrizEscalona = Matriz::CriarMatrizTransformacaoEscalonamento(escalona);
  Matriz* transformacao = Matriz::AdicionaTransformacao(matrizTransladaCentro, matrizEscalona, true);
  translada[0]= -translada[0];
  translada[1]= -translada[1];
  translada[2]= -translada[2];
  Matriz* matrizTransladaDeVolta =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
  transformacao = Matriz::AdicionaTransformacao(transformacao, matrizTransladaDeVolta, true);

  AplicaTransformacao(transformacao, false);
  delete transformacao;
  delete centro;
  return true;
}
//---------------------------------------------------------------------------
bool Window::ZoomOut(float passo) // Aumenta o tamanho do window
{

  if (GetXMaxWindow()-GetXMinWindow()>maximo ||
    GetYMaxWindow()-GetYMinWindow()>maximo)
    return false;

  // Pega o numero de pixels a deslocar em funcao da porcentagem de passo
  float porcent = passo/100;
  float pixelsX = (GetXMaxWindow()-GetXMinWindow())*porcent;
  float pixelsY = (GetYMaxWindow()-GetYMinWindow())*porcent;
  pixelsX = pixelsX == 0 ? 1 : pixelsX; // Pode acontecer de o numero calculado ser 0
  pixelsY = pixelsY == 0 ? 1 : pixelsY;

  SetXMaxWindow(GetXMaxWindow() + pixelsX);
  SetYMaxWindow(GetYMaxWindow() + pixelsY);
  SetXMinWindow(GetXMinWindow() - pixelsX);
  SetYMinWindow(GetYMinWindow() - pixelsY);

  // Atualiza as coordenadas de mundo
  float escalona[3];
  escalona[0] = 1+ 2*porcent;
  escalona[1] = 1+ 2*porcent;
  escalona[2] = 1+ 2*porcent;
  float translada[3];
  Ponto* centro = Centro();
  translada[0] = -centro->GetX();
  translada[1] = -centro->GetY();
  translada[2] = -centro->GetZ();

  Matriz* matrizTransladaCentro =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
  Matriz* matrizEscalona = Matriz::CriarMatrizTransformacaoEscalonamento(escalona);
  Matriz* transformacao = Matriz::AdicionaTransformacao(matrizTransladaCentro, matrizEscalona, true);
  translada[0]= -translada[0];
  translada[1]= -translada[1];
  translada[2]= -translada[2];
  Matriz* matrizTransladaDeVolta =  Matriz::CriarMatrizTransformacaoTranslacao(translada);
  transformacao = Matriz::AdicionaTransformacao(transformacao, matrizTransladaDeVolta, true);

  AplicaTransformacao(transformacao, false);
  delete transformacao;
  delete centro;
  return true;
}
//---------------------------------------------------------------------------
void Window::Mover(float passo, int d){ // Desloca o window

  // Pega o numero de pixels a deslocar em funcao da porcentagem de passo
  float pixelsX = (GetXMaxWindow()-GetXMinWindow())*(float)passo/100;
  float pixelsY = (GetYMaxWindow()-GetYMinWindow())*(float)passo/100;
  switch (d){
    case DIREITA:
      SetXMinWindow(GetXMinWindow() + pixelsX);
      SetXMaxWindow(GetXMaxWindow() + pixelsX);
      break;
    case ESQUERDA:
      SetXMinWindow(GetXMinWindow() - pixelsX);
      SetXMaxWindow(GetXMaxWindow() - pixelsX);
      break;
    case CIMA:
      SetYMinWindow(GetYMinWindow() + pixelsY);
      SetYMaxWindow(GetYMaxWindow() + pixelsY);
      break;
    case BAIXO:
      SetYMinWindow(GetYMinWindow() - pixelsY);
      SetYMaxWindow(GetYMaxWindow() - pixelsY);
      break;
  }

  // Translada a window conforme o angulo dela
  Matriz* matrizX = Matriz::CriarMatrizTransformacaoRotacaoX(anguloY-90);
  Matriz* matrizY = Matriz::CriarMatrizTransformacaoRotacaoY(anguloX-90);
  Matriz* matrizZ = Matriz::CriarMatrizTransformacaoRotacaoZ(angulo);
  Matriz* transformacao;
  Ponto* p;
  switch(d) {
    case CIMA:
      p = new Ponto("TESTE", 0.0,pixelsY,0.0);
      transformacao = Matriz::AdicionaTransformacao(matrizX, matrizY, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
    break;
    case BAIXO:
      p = new Ponto("TESTE", 0.0,-pixelsY,0.0);
      transformacao = Matriz::AdicionaTransformacao(matrizX, matrizY, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
    break;
    case ESQUERDA:
      p = new Ponto("TESTE", -pixelsX,0.0,0.0);
      transformacao = Matriz::AdicionaTransformacao(matrizY, matrizX, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
    break;
    case DIREITA:
      transformacao = Matriz::AdicionaTransformacao(matrizY, matrizX, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
      p = new Ponto("TESTE", pixelsX,0.0,0.0);
    break;
    case FRENTE:
      p = new Ponto("TESTE", 0.0,0.0,pixelsX);
      transformacao = Matriz::AdicionaTransformacao(matrizX, matrizY, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
    break;
    case TRAZ:
      p = new Ponto("TESTE", 0.0,0.0,-pixelsX);
      transformacao = Matriz::AdicionaTransformacao(matrizX, matrizY, true);
      transformacao = Matriz::AdicionaTransformacao(transformacao, matrizZ, true);
    break;
  }

  p->AplicaTransformacao(transformacao,false);

  float translacao[3];
  translacao[0] = p->GetX();
  translacao[1] = p->GetY();
  translacao[2] = p->GetZ();
  Matriz* matriz = Matriz::CriarMatrizTransformacaoTranslacao(translacao);
  AplicaTransformacao(matriz, false);
  delete matriz;
  delete p;
  delete transformacao;
}

//---------------------------------------------------------------------------
void Window::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  WireFrame::AplicaTransformacao(matrizDeTransformacao, CPP);
  AplicaTransformacaoNoPonto(COP, matrizDeTransformacao, CPP);
}
//---------------------------------------------------------------------------
void Window::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao,bool CPP)
{
  WireFrame::AplicaTransformacao(translacao, escalonamento, rotacao, CPP);
  COP->AplicaTransformacao(translacao, escalonamento, rotacao,CPP);
}
//---------------------------------------------------------------------------

Ponto& Window::GetCOP()
{
  return *COP;
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
