#pragma hdrstop

#include "WireFrame.h"
#include "Ponto.h"
#include "Reta.h"

//---------------------------------------------------------------------------
WireFrame::WireFrame(string nome)
 : ObjetoGrafico(nome)
{
}
//---------------------------------------------------------------------------
WireFrame::~WireFrame()
{
  for(unsigned int i=0;i<pontos.size();i++)
    delete pontos[i];
}
//---------------------------------------------------------------------------
void WireFrame::AdicionaPonto(Ponto* p)
{
  pontos.push_back(p);
}
//---------------------------------------------------------------------------
void WireFrame::AdicionaAresta(Reta* r)
{
  arestas.push_back(r);
}
//---------------------------------------------------------------------------
vector<Ponto*>* WireFrame::RetornaPontos()
{
  return &pontos;
}
//---------------------------------------------------------------------------
int WireFrame::NrPontos()
{
  return pontos.size();
}
//---------------------------------------------------------------------------
Ponto* WireFrame::Centro()
{
  float x = 0;
  float y = 0;
  float z = 0;
  for(unsigned int i=0;i<pontos.size();i++){
    x += pontos[i]->GetX();
    y += pontos[i]->GetY();
    z += pontos[i]->GetZ();
  }
  x = x / pontos.size();
  y = y / pontos.size();
  z = z / pontos.size();

  return new Ponto("Centro",x,y,z);
}
//---------------------------------------------------------------------------
void WireFrame::AplicaTransformacao(float* translacao, float* escalonamento, float* rotacao, bool CPP)
{
    Ponto* centro = NULL;
    if (!CPP)
      centro = Centro();
    Matriz* matriz = CriarMatrizTransformacoes(translacao,escalonamento,rotacao,centro);
    for (unsigned int i=0;i<pontos.size();i++) {
      Ponto* p = pontos[i];
      AplicaTransformacaoNoPonto(p, matriz,CPP);
    }
    delete matriz;
    delete centro;
}
//---------------------------------------------------------------------------
void WireFrame::AplicaTransformacao(Matriz* matrizDeTransformacao, bool CPP)
{
  for (unsigned int i=0;i<pontos.size();i++) {
    Ponto* p = pontos[i];
    AplicaTransformacaoNoPonto(p, matrizDeTransformacao,CPP);
  }
}
//---------------------------------------------------------------------------
void WireFrame::SalvaNoStream(ostream &output)
{
  ObjetoGrafico::SalvaNoStream(output);
  output << pontos.size() << " ";
  for (unsigned int i=0;i<pontos.size();i++)
    pontos[i]->SalvaNoStream(output);

  output << arestas.size() << " ";
  for (unsigned int i=0;i<arestas.size();i++) {
    output << arestas[i]->GetP1().GetHandle() << " ";
    output << arestas[i]->GetP2().GetHandle() << " ";
  }

}
//---------------------------------------------------------------------------
WireFrame* WireFrame::LeDoStream(istream &input, int versao)
{
  string nome = ObjetoGrafico::LeNomeDoStream(input);
  WireFrame* wireframe = new WireFrame(nome);
  int nrPontos;
  input >> nrPontos;
  for(int i=0 ;i<nrPontos;i++)
    wireframe->AdicionaPonto(Ponto::LeDoStream(input, versao));
  int nrArestas;
  input >> nrArestas;
  for(int i=0 ;i<nrArestas;i++) {
    vector<Ponto*>* pontosTemp = wireframe->RetornaPontos();
    unsigned int handle1, handle2;
    input >> handle1 >> handle2;
    Reta* r = new Reta("");
    for (unsigned int j=0;j<pontosTemp->size();j++) {
      if(handle1 == (*pontosTemp)[j]->GetHandle())
        r->SetP1((*pontosTemp)[j]);
      if(handle2 == (*pontosTemp)[j]->GetHandle())
        r->SetP2((*pontosTemp)[j]);
    }
    wireframe->AdicionaAresta(r);
  }

  return wireframe;
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
